//
// Controls.h
//

#ifndef _BaseControls_H
#define _BaseControls_H

#include <KCL.h>

// Be includes
#include <Beep.h>
#include <Point.h>
#include <View.h>
#include <Button.h>
#include <StringView.h>
#include <Box.h>
#include <CheckBox.h>
#include <RadioButton.h>
#include <ListView.h>
#include <OutlineListView.h>
#include <Slider.h>
#include <ColorControl.h>
#include <StringView.h>
#include <TextControl.h>
#include <MenuField.h>
#include <StatusBar.h>
#include <TextView.h>
#include <ScrollView.h>

void SetViewVisible(BView *v, bool b);

template <class T>
class TView: public BView, public TViewEventSource<T> {
	typedef BView inherited;
public:
	
	TView<T>(
		T *owner
	, BRect frame
	, const char *name
	, uint32 resizeMode
	, uint32 flags
	)
	: BView(frame, name, resizeMode, flags)
	, TViewEventSource<T>(owner)
	{}
	
	// BView overrides
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		DoMouseUp(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		DoMouseMoved(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
	
};


//
// Button
//
template <class T> 
class TButton: public BButton, public TViewEventSource<T> {
	typedef BButton inherited;
public:
	
	typedef void (T::*TInvokedEvent)(void);

	TButton<T>(
		T* owner
	,	BRect frame
	, const char *name
  , const char *label
  , BMessage *message = NULL
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE
  )
  : BButton(frame, name, label, message, resizingMode, flags)
  , TViewEventSource<T>(owner)
  , OnInvoked(0)
  {}
  
  virtual void SetDefault(bool bDefault) { MakeDefault(bDefault);}
  virtual bool Default(void) const { return IsDefault(); }
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
  
  TInvokedEvent OnInvoked;
  
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnInvoked) {
  		(EventObserver()->*OnInvoked)();
  	}
  	return result;
  }
  
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		DoMouseUp(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		DoMouseMoved(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// Box
//
template <class T> 
class TBox: public BBox, public TViewEventSource<T> {
	typedef BBox inherited;
public:
	
	TBox<T>(
		T* owner
	,	BRect frame
	, const char *name
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP
  , border_style border = B_FANCY_BORDER
  )
  : BBox(frame, name, resizingMode, flags, border)
  , TViewEventSource<T>(owner)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 

	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		DoMouseUp(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		DoMouseMoved(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// CheckBox
//
template <class T> 
class TCheckBox: public BCheckBox, public TViewEventSource<T> {
	typedef BCheckBox inherited;
public:

	typedef void (T::*TCheckedEvent)(bool);
	
	TCheckBox<T>(
		T* owner
	,	BRect frame
	, const char *name
	, const char *label
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE
  )
  : BCheckBox(frame, name, label, 0, resizingMode, flags)
  , TViewEventSource<T>(owner)
  , OnChecked(0)
  {}
  
  // properties
  bool Checked(void) const { return(Value()!=0); }
  void SetChecked(bool b) { SetValue(b?1:0); }
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 

  // events
  TCheckedEvent OnChecked;
  
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnChecked) 
  		(EventObserver()->*OnChecked)(Value());
  	return result;
  }
  
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		DoMouseUp(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		DoMouseMoved(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// RadioButton
//
template <class T> 
class TRadioButton: public BRadioButton, public TViewEventSource<T> {
	typedef BRadioButton inherited;
public:

	typedef void (T::*TCheckedEvent)(bool);
	
	TRadioButton<T>(
		T* owner
	,	BRect frame
	, const char *name
	, const char *label
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE
  )
  : BRadioButton(frame, name, label, 0, resizingMode, flags)
  , TViewEventSource<T>(owner)
  , OnChecked(0)
  {}
  
  // properties
  bool Checked(void) const { return(Value()!=0); }
  void SetChecked(bool b) { SetValue(b?1:0); }
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
  
  // events
  TCheckedEvent OnChecked;
  
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnChecked) 
  		(EventObserver()->*OnChecked)(Value());
  	return result;
  }
  
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		DoMouseUp(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		DoMouseMoved(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// ListView
//
template <class T> 
class TListView: public BListView, public TViewEventSource<T> {
	typedef BListView inherited;
public:
	typedef void (T::*TItemSelectedEvent)(int32);
	typedef void (T::*TItemInvokedEvent)(int32);

	TListView<T>(
		T* owner
	,	BRect frame
	, const char *name
	, list_view_type type = B_SINGLE_SELECTION_LIST
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS
  )
  : BListView(frame, name, type, resizingMode, flags)
  , TViewEventSource<T>(owner)
  , OnItemSelected(0)
  , OnItemInvoked(0)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 

  // events
  TItemSelectedEvent OnItemSelected;
  TItemInvokedEvent OnItemInvoked;
  
  virtual void SelectionChanged(void)
  {
  	BListView::SelectionChanged();
  	if(OnItemSelected) (EventObserver()->*OnItemSelected)(CurrentSelection());
  }
  
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = BListView::Invoke(msg);
  	
  	if(OnItemInvoked) (EventObserver()->*OnItemInvoked)(CurrentSelection());
  	
  	return result;
  }

	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// ListView
//
template <class T> 
class TOutlineListView: public BOutlineListView, public TViewEventSource<T> {
	typedef BOutlineListView inherited;
public:
	typedef void (T::*TItemSelectedEvent)(int32);
	typedef void (T::*TItemInvokedEvent)(int32);

	TOutlineListView<T>(
		T* owner
	,	BRect frame
	, const char *name
	, list_view_type type = B_SINGLE_SELECTION_LIST
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS
  )
  : BOutlineListView(frame, name, type, resizingMode, flags)
  , TViewEventSource<T>(owner)
  , OnItemSelected(0)
  , OnItemInvoked(0)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 

  // events
  TItemSelectedEvent OnItemSelected;
  TItemInvokedEvent OnItemInvoked;
  
  virtual void SelectionChanged(void)
  {
  	inherited::SelectionChanged();
  	if(OnItemSelected) (EventObserver()->*OnItemSelected)(CurrentSelection());
  }
  
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	
  	if(OnItemInvoked) (EventObserver()->*OnItemInvoked)(CurrentSelection());
  	
  	return result;
  }

	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// Slider
//
template <class T> 
class TSlider: public BSlider, public TViewEventSource<T> {
	typedef BSlider inherited;
public:
	typedef void (T::*TSliderChangeEvent)(void);
	typedef void (T::*TDrawBarEvent)(bool &bDefaultDraw);
	typedef void (T::*TDrawThumbEvent)(bool &bDefaultDraw);
	typedef void (T::*TDrawFocusMarkEvent)(bool &bDefaultDraw);
	typedef void (T::*TDrawHashMarkEvent)(bool &bDefaultDraw);
	typedef void (T::*TDrawTextEvent)(bool &bDefaultDraw);
	typedef void (T::*TDrawSliderEvent)(bool &bDefaultDraw);

	TSlider<T>(
		T* owner
	,	BRect frame
	, const char *name
	, const char *label
	, BMessage *message
	, int32 minvalue
	, int32 maxvalue
	, thumb_style thumbType = B_BLOCK_THUMB
  , uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS
  )
  : BSlider(frame, name, label, message, minvalue, maxvalue, thumbType, resizingMode, flags)
  , TViewEventSource<T>(owner)
 	, OnSliderChange(0)
	,	OnDrawBar(0)
	,	OnDrawThumb(0)
	,	OnDrawFocusMark(0)
	,	OnDrawHashMarks(0)
	,	OnDrawText(0)
	,	OnDrawSlider(0)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	void SetMinValueLabel(const char *lbl)	{	SetLimitLabels(lbl, MaxLimitLabel());	}
	void SetMaxValueLabel(const char *lbl)	{	SetLimitLabels(MinLimitLabel(), lbl);	}

  // events
	TSliderChangeEvent 	OnSliderChange;
	TDrawBarEvent				OnDrawBar;
	TDrawThumbEvent			OnDrawThumb;
	TDrawFocusMarkEvent	OnDrawFocusMark;
	TDrawHashMarkEvent	OnDrawHashMarks;
	TDrawTextEvent			OnDrawText;
	TDrawSliderEvent		OnDrawSlider;
	
	// slider events
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnSliderChange) {
  		(EventObserver()->*OnSliderChange)();
  	}
  	return result;
  }

  virtual void DrawBar(void)
  {
  	if(OnDrawBar) 
  	{
  		bool bDefault = false;
			(EventObserver()->*OnDrawBar)(bDefault);
			if(bDefault) 
				inherited::DrawBar();
  	}
  	else
  		inherited::DrawBar();
  }

  virtual void DrawThumb(void)
  {
  	if(OnDrawThumb) 
  	{
  		bool bDefault = false;
			(EventObserver()->*OnDrawThumb)(bDefault);
			if(bDefault) 
				inherited::DrawThumb();
  	}
  	else
  		inherited::DrawThumb();
  }

  virtual void DrawFocusMark(void)
  {
  	if(OnDrawFocusMark) 
  	{
  		bool bDefault = false;
			(EventObserver()->*OnDrawFocusMark)(bDefault);
			if(bDefault) 
				inherited::DrawFocusMark();
  	}
  	else
  		inherited::DrawFocusMark();
  }

  virtual void DrawHashMarks(void)
  {
  	if(OnDrawHashMarks) 
  	{
  		bool bDefault = false;
			(EventObserver()->*OnDrawHashMarks)(bDefault);
			if(bDefault) 
				inherited::DrawHashMarks();
  	}
  	else
  		inherited::DrawHashMarks();
  }

  virtual void DrawText(void)
  {
  	if(OnDrawText) 
  	{
  		bool bDefault = false;
			(EventObserver()->*OnDrawText)(bDefault);
			if(bDefault) 
				inherited::DrawText();
  	}
  	else
  		inherited::DrawText();
  }

  virtual void DrawSlider(void)
  {
  	if(OnDrawSlider) 
  	{
  		bool bDefault = false;
			(EventObserver()->*OnDrawSlider)(bDefault);
			if(bDefault) 
				inherited::DrawSlider();
  	}
  	else
  		inherited::DrawSlider();
  }

	// regular view events
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// ColorControl
//
template <class T> 
class TColorControl: public BColorControl, public TViewEventSource<T> {
	typedef BColorControl inherited;
public:
	
	typedef void (T::*TChangeEvent)(void);

	TColorControl<T>(
		T* owner
	,	BRect frame
	, const char *name
  , color_control_layout layout
  , float cellSize
  , bool bufferedDrawing = false
  , BMessage *message = NULL
  )
  : BColorControl(frame.LeftTop(), layout, cellSize, name, message, bufferedDrawing)
  , TViewEventSource<T>(owner)
  , OnChange(0)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	void SetColor(rgb_color clr) { SetValue(clr); }
	rgb_color Color(void) const { return ValueAsColor(); }
  
  TChangeEvent OnChange;
  
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnChange) {
  		(EventObserver()->*OnChange)();
  	}
  	return result;
  }
  
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// StringView
//
template <class T> 
class TStringView: public BStringView, public TViewEventSource<T> {
	typedef BStringView inherited;
public:
	
	TStringView<T>(
		T* owner
	,	BRect frame
	, const char *name
  , const char *label
  , uint32 resizeMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW
  )
  : BStringView(frame, name, label, resizeMode, flags)
  , TViewEventSource<T>(owner)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// TextControl
//
template <class T> 
class TTextControl: public BTextControl, public TViewEventSource<T> {
	typedef BTextControl inherited;
public:

	typedef void (T::*TChangedEvent)(const char *);
	typedef void (T::*TFocusEvent)(void);
	
	TTextControl<T>(
		T* owner
	,	BRect frame
	, const char *name
  , const char *label
  , const char *text
  , uint32 resizeMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW
  )
  : BTextControl(frame, name, label, text, 0, resizeMode, flags)
  , TViewEventSource<T>(owner)
  , OnChanged(0)
  {}
  
  TChangedEvent OnChanged;
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	alignment LabelAlignment(void) const
	{ 
		alignment aeText, aeLabel;
		GetAlignment(&aeLabel, &aeText);
		return aeLabel;
	}
	
	void SetLabelAlignment(alignment a)
	{
		alignment aeText, aeLabel;
		GetAlignment(&aeLabel, &aeText);
		
		aeLabel = a;
		
		if(LockLooper())
		{
			SetAlignment(aeLabel, aeText);
			UnlockLooper();
		}
	}
	
	alignment TextAlignment(void) const
	{
		alignment aeText, aeLabel;
		GetAlignment(&aeLabel, &aeText);
		return aeText;
	}
	
	void SetTextAlignment(alignment a)
	{
		alignment aeText, aeLabel;
		GetAlignment(&aeLabel, &aeText);
		
		aeText = a;
		
		if(LockLooper())
		{
			SetAlignment(aeLabel, aeText);
			UnlockLooper();
		}
	}
	
	// events 
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnChanged) {
  		(EventObserver()->*OnChanged)(Text());
  	}
  	return result;
  }
	
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(TextView()->IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// TextView
//
template <class T> 
class TTextView: public BTextView, public TViewEventSource<T> {
	typedef BTextView inherited;
public:
	typedef void (T::*TChangedEvent)(const char *);

	TTextView<T>(
		T* owner
	,	BRect frame
	, const char *name
  , const char *text
  , uint32 resizeMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW
  )
  : BTextView(frame, name, BRect(4,4,frame.Width()-4,frame.Height()-4), resizeMode, flags)
  , TViewEventSource<T>(owner)
  , OnChanged(0)
  {}
  
  TChangedEvent OnChanged;
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	// events 
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// MenuField
//
template <class T> 
class TMenuField: public BMenuField, public TViewEventSource<T> {
	BMenu *m_Menu;
	
	typedef BMenuField inherited;
public:

	TMenuField<T>(
		T* owner
	,	BRect frame
	, const char *name
  , const char *label
  , uint32 resizeMode = B_FOLLOW_LEFT | B_FOLLOW_TOP
  , uint32 flags = B_WILL_DRAW | B_NAVIGABLE
  )
  : BMenuField(frame, name, label, (m_Menu = new BMenu(label)), resizeMode, flags)
  , TViewEventSource<T>(owner)
  {}
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	// properties
	virtual void SetTriggersEnabled(bool b) {
		Menu()->SetTriggersEnabled(b);
	}
	
	virtual void SetMaxContentWidth(float width) {
		Menu()->SetMaxContentWidth(width);
	}
	
	// events 
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// StatusBar
//
template <class T> 
class TStatusBar: public BStatusBar, public TViewEventSource<T> {
	typedef BStatusBar inherited;
public:

	typedef void (T::*TResetEvent)(void);
	
	TStatusBar<T>(
		T* owner
	,	BRect frame
	, const char *name
  , const char *label = NULL
  , const char *trailinglabel = NULL
  )
  : BStatusBar(frame, name, label, trailinglabel)
  , TViewEventSource<T>(owner)
  , OnReset(0)
  {}
  
  TResetEvent OnReset;
  
  void SetVisible(bool b) { SetViewVisible(this, b); }
	bool Visible(void) const { return(!IsHidden()); } 
	
	// events
	virtual void Reset(const char *label = NULL, const char *trailinglabel = NULL)
	{
		inherited::Reset(label, trailinglabel);
		if(OnReset) {
			(EventObserver()->*OnReset)();
		}
	}
	
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	Draw(BRect updateRect)
	{
		if(IsFocusChanging())	{
			if(IsFocus())	{
				if(OnGotFocus) {
					(EventObserver()->*OnGotFocus)();
				}
			}	else {
				if(OnLostFocus) {
					(EventObserver()->*OnLostFocus)();
				}
			}
		}
		inherited::Draw(updateRect);
		DoDraw(updateRect);
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual	void	TargetedByScrollView(BScrollView *scroll_view)
	{
		inherited::TargetedByScrollView(scroll_view);
		DoTargetedByScrollView(scroll_view);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

//
// StatusBar
//
template <class T> 
class TScrollView: public BScrollView, public TViewEventSource<T> {
	typedef BScrollView inherited;
public:

	typedef void (T::*TResetEvent)(void);
	
	TScrollView<T>(
		T* owner
	, const char *name
  , BView *target
  , uint32 resizeMask = B_FOLLOW_LEFT|B_FOLLOW_TOP
  , uint32 flags = 0
  , bool horz = false
  , bool vert = false
  , border_style border = B_FANCY_BORDER
  )
  : BScrollView(name, target, resizeMask, flags, horz, vert, border)
  , TViewEventSource<T>(owner)
  {}
  
	// events
	virtual	void	AttachedToWindow()
	{
		inherited::AttachedToWindow();
		DoAttachedToWindow();
	}

	virtual	void	AllAttached()
	{
		inherited::AllAttached();
		DoAllAttached();
	}

	virtual	void	DetachedFromWindow()
	{
		inherited::DetachedFromWindow();
		DoDetachedFromWindow();
	}
	
	virtual	void	AllDetached()
	{
		inherited::AllDetached();
		DoAllDetached();
	}
	
	virtual	void	MouseDown(BPoint where)
	{
		inherited::MouseDown(where);
		DoMouseDown(where);
	}

	virtual	void	MouseUp(BPoint where)
	{
		inherited::MouseUp(where);
		if(OnMouseUp) (EventObserver()->*OnMouseUp)(where);
	}
	
	virtual	void	MouseMoved(BPoint where,uint32 code,const BMessage *a_message)
	{
		inherited::MouseMoved(where, code, a_message);
		if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message);
	}
	
	virtual	void	WindowActivated(bool state)
	{
		inherited::WindowActivated(state);
		DoWindowActivated(state);
	}
	
	virtual	void	KeyDown(const char *bytes, int32 numBytes)
	{
		inherited::KeyDown(bytes, numBytes);
		DoKeyDown(bytes, numBytes);
	}
	
	virtual	void	KeyUp(const char *bytes, int32 numBytes)
	{
		inherited::KeyUp(bytes, numBytes);
		DoKeyUp(bytes, numBytes);
	}
	
	virtual	void	Pulse()
	{
		inherited::Pulse();
		DoPulse();
	}
	
	virtual	void	FrameMoved(BPoint new_position)
	{
		inherited::FrameMoved(new_position);
		DoFrameMoved(new_position);
	}
	
	virtual	void	FrameResized(float new_width, float new_height)
	{
		inherited::FrameResized(new_width, new_height);
		DoFrameResized(new_width, new_height);
	}
	
	virtual void  SetFont(const BFont *font, uint32 mask = B_FONT_ALL)
	{
		inherited::SetFont(font,mask);
		DoSetFont(font,mask);
	}
};

#endif //_BaseControls_H