//
// ComponentDesign.cpp
//

#include "ComponentDesign.h"
#include <stdlib.h>
#include <stdio.h>
#include <string>

#include <ClassInfo.h>
#include <Beep.h>



//
// Private classes
//
class TLaunchFlagsProperty: public TEnumProperty {
public:
	TLaunchFlagsProperty(TComponent *component, TProperty *prop)
	: TEnumProperty(component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->Add("B_SINGLE_LAUNCH");
		list->Add("B_MULTIPLE_LAUNCH");
		list->Add("B_EXCLUSIVE_LAUNCH");
	}
};


//
// TPropertyEditor implementation
//
void TPropertyEditor::SetValue(const char *value, bool setControlToo = true)
{
	if(m_Component)
	{
		if(m_Component->PropertyModified(m_Prop->Name(), m_Prop->Value(), value))
		{
			if(m_Component->Owner())
			{
				if(m_Component->Owner()->ComponentPropertyModified(
					m_Component
				, m_Prop->Name()
				, m_Prop->Value()
				, value
				))
				{
					m_Prop->SetValue(value);
					if(setControlToo)
					{
						m_Component->SetPropertyValue(m_Prop->Name(), value);
					}
				}
			}
			else
			{
				m_Prop->SetValue(value);
				if(setControlToo)
				{
					m_Component->SetPropertyValue(m_Prop->Name(), value);
				}
			}
		}
	}
	else
	{
		m_Prop->SetValue(value);
	}
}

//
// TEvent implementation
//
TEvent::TEvent(
	const char *name
, const char *params
, const char *rettype
, bool selected
)
: m_Name(name)
, m_Params(params)
, m_RetType(rettype)
, m_Selected(selected)
{	}


//
// TPropertyStore implementation
//
TPropertyStore::TPropertyStore(TComponent *component) 
: BMessage('prop') 
, m_Component(component)
{}

TPropertyStore::~TPropertyStore() {}

status_t TPropertyStore::SetBool(const char *name, bool b) {
	if (HasBool(name)) {
		return ReplaceBool(name, 0, b);
	}
	return AddBool(name, b);
}

status_t TPropertyStore::SetInt8(const char *name, int8 i) {
	if (HasInt8(name)) {
		return ReplaceInt8(name, 0, i);
	}
	return AddInt8(name, i);
}

status_t TPropertyStore::SetInt16(const char *name, int16 i) {
	if (HasInt16(name)) {
		return ReplaceInt16(name, 0, i);
	}
	return AddInt16(name, i);
}

status_t TPropertyStore::SetInt32(const char *name, int32 i) {
	if (HasInt32(name)) {
		return ReplaceInt32(name, 0, i);
	}
	return AddInt32(name, i);
}

status_t TPropertyStore::SetInt64(const char *name, int64 i) {
	if (HasInt64(name)) {
		return ReplaceInt64(name, 0, i);
	}
	return AddInt64(name, i);
}

status_t TPropertyStore::SetFloat(const char *name, float f) {
	if (HasFloat(name)) {
		return ReplaceFloat(name, 0, f);
	}
	return AddFloat(name, f);
}

status_t TPropertyStore::SetDouble(const char *name, double f) {
	if (HasDouble(name)) {
		return ReplaceDouble(name, 0, f);
	}
	return AddDouble(name, f);
}

status_t TPropertyStore::SetString(const char *name, const char *s) {
	const char *buf = s;
	if(!buf) buf = "";
	
	if (HasString(name)) {
		return ReplaceString(name, 0, buf);
	}
	return AddString(name, buf);
}

status_t TPropertyStore::SetPoint(const char *name, BPoint p) {
	if (HasPoint(name)) {
		return ReplacePoint(name, 0, p);
	}
	return AddPoint(name, p);
}

status_t TPropertyStore::SetRect(const char *name, BRect r) {
	if (HasRect(name)) {
		return ReplaceRect(name, 0, r);
	}
	return AddRect(name, r);
}

status_t TPropertyStore::SetMessage(const char *name, const BMessage *message) {
	if (HasMessage(name)) {
		return ReplaceMessage(name, 0, message);
	}
	return AddMessage(name, message);
}

status_t TPropertyStore::SetFlat(const char *name, const BFlattenable *obj) {
	if (HasFlat(name, obj)) {
		return ReplaceFlat(name, 0, (BFlattenable *) obj);
	}
	return AddFlat(name, (BFlattenable *) obj);
}

//
// TProperty implementation
//
const char *TProperty::GetStatement(void) const
{
	char buf[T_PROP_VALUE_LENGTH + 80] = "";	
	if(!strlen(m_Get))
	{
		sprintf((char *)buf, "%s->%s()", m_Owner->ComponentName(), Name());
	}
	else
	{
		strcpy(buf, m_Get);
	}
	return buf;
}

const char *TProperty::SetStatement(void) const
{
	char buf[T_PROP_VALUE_LENGTH + 80] = "";
	if(!strlen(m_Set))
	{
		if(IsQuoted())
			sprintf((char *)buf, "%s->Set%s(\"%s\");", m_Owner->ComponentName(), Name(), Value());
		else
			sprintf((char *)buf, "%s->Set%s(%s);", m_Owner->ComponentName(), Name(), Value());
	}
	else
	{
		strcpy(buf, m_Set);
	}
	return buf;
}


//
// TComponent implementation
//
// Put all of the stored values into the properties and their editors.
void TComponent::InitPropertyValues(void)
{
	for(int i = 0; i < CountProperties(); i++)
	{
		if(PropertyAt(i)->Editor())
		{
			PropertyAt(i)->Editor()->SetValue(
				GetPropertyValue(PropertyAt(i)->Name())
			);
		}
	}
}

// Initialize design-time event values
void TComponent::InitEventValues(void)
{
}

void TComponent::InitPropertyList(void)
{
	// Clear properties
	PropertyStore()->MakeEmpty();
	ClearProperties();
	
	// Load properties for all TComponents
	AddPropAndEditor(
		new TProperty(this, "Name", ComponentName(), 0, false)
	, new TNameProperty(this, 0)
	, 0, 0, true
	);
}

const char *TComponent::GetPropertyValue(const char *propName) const
{
	printf("TComponent::GetPropertyValue('%s')\n", propName);
	
	string s = propName;
	if(s == "Name")
	{
		return ComponentName();
	}
	
	const char *buf = 0;
	if(B_OK == PropertyStore()->FindString(propName, &buf))
		return buf;
	else
		return NULL; //"(No value)";
}

void TComponent::StoreProperty(const char *name, const char *value)
{
	BString oldValue;
	
	if((!name) || (!value)) return;
	
	// Find the property associated with the name
	TProperty *prop = FindProperty(name);
	if(prop) 
	{
		// save the old value
		oldValue = prop->Value();
		
		// Notify any listeners for this property
		if(CountPropertyListeners() > 0)
		{
			for(int32 i = 0; i < CountPropertyListeners(); i++)
			{
				TPropertyEditor *e = PropertyListenerAt(i);
				if(e)
				{
					e->PropListenerNotify(this, name, oldValue.String(), value);
				}	
			}
		}

		// If the new and old values are different,
		if((prop->Value()) && (strcmp(prop->Value(), value)))
		{
			// If the property is allowed to be modified
			if(PropertyModified(name, prop->Value(), value))
			{
				// If the owner exists and allows the modification
				if(Owner())
				{
					if(Owner()->ComponentPropertyModified(this, name, prop->Value(), value))
					{	
						SetModified(true);
					}
				} 
				else // no owner
				{
					SetModified(true);
				}
			}
			else
				return;
		}
		prop->SetValue(value);
		PropertyStore()->SetString(name, value);
	}
	else // no property, just store the value
	{
		PropertyStore()->SetString(name, value);
	}
}

void TComponent::SetPropertyValue(const char *propName, const char *value)
{
	printf("TComponent::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	if(0 == strcmp("Name", propName))	
	{
		SetComponentName(value);
	}
	StoreProperty(propName, value);
}

void TComponent::InitEventList(void)
{
	// Load default events for TComponents (there really aren't any)
	ClearEvents();
}

bool TComponent::EventSelection(const char *name) const
{
	TEvent *evt = (const_cast<TComponent *>(this))->FindEvent(name);
	if(evt)
	{
		return evt->IsSelected();
	}
	return false;
}

void TComponent::SetEventSelection(const char *name, bool selected)
{
	TEvent *evt = FindEvent(name);
	if(evt)
	{
		evt->SetSelected(selected);
	}
}

// Events from the design-time environment
void TComponent::DroppedOnFormDesigner(BRect location) {}
void TComponent::SelectedInFormDesigner(void) {}
void TComponent::InvokedInFormDesigner(void) {}
void TComponent::PopupMenuInvokedInFormDesigner(BPopUpMenu *menu) {}
void TComponent::MenuItemSelectedInFormDesigner(BMenuItem *item) {}
void TComponent::AddedToPalette(void) {}
void TComponent::PopupMenuInvokedInPalette(BPopUpMenu *menu) {}
void TComponent::MenuItemSelectedInPalette(BMenuItem *item) {}
void TComponent::DeletedFromPalette(void) {}

void TComponent::DeletedFromFormDesigner(void) 
{
	// Make sure any component-type property editors on the form
	// designer lose their references to this component!
	for(int32 i = 0; i < CountPropertyListeners(); i++)
	{
		TPropertyEditor *ed = PropertyListenerAt(i);
		if(ed)
		{
			TComponentProperty *cp = dynamic_cast<TComponentProperty *>(ed);
			if(cp)
			{
				cp->SetComponentRef(NULL);
			}
		}
	}
}

void TComponent::DesignerMouseDown(BPoint pt, BRect bounds, BView *view)
{
	int32 i;
	
	if(!IsDesignerSelected())
	{
		DesignerSelect(true);
	
		if(modifiers() & B_SHIFT_KEY)
		{
			// The rule is:
			// If the shift key is down and this control was selected, keep the selection of 
			// all controls within this control's parent.
		}
		else
		{
			if(Owner())
			{
				for(i = 0; i < Owner()->CountComponents(); i++)
				{
					if(Owner()->ComponentAt(i) && Owner()->ComponentAt(i) != this)
					{
						Owner()->ComponentAt(i)->DesignerSelect(false);
						if(Owner()->ComponentAt(i)->DTView())
							Owner()->ComponentAt(i)->DTView()->Invalidate();
					}
				}
			}
		}
			
		DTView()->Invalidate();
		for(i = 0; i < DTView()->CountChildren(); i++)
		{
			BView *vw = DTView()->ChildAt(i);
			if(vw) vw->Invalidate();
		}
		
		if(Designer())
		{
			SelectedInFormDesigner();
			Designer()->ComponentSelected(this);
		}
	}
	else // is selected in designer
	{
		if(modifiers() & B_SHIFT_KEY)
		{
			// If this control is being clicked while the shift key is down, it means
			// to take it out of the selection.
			DesignerSelect(false);
			Designer()->ComponentSelected(this, false);
			return;
		}
	}
	
	// If the user double-clicked on the control, 
	// invoke the default edit action if there is one.
	BMessage *msg = view->Window()->CurrentMessage();
	if(msg)
	{
		int32 clicks;
		if(B_OK == msg->FindInt32("clicks", &clicks))
		{
			if(clicks > 1)
			{
				if(DefaultEditAction(pt))
				{
					
				}
				return;
			}
		}
	}
	
	// If this is a parent control, 
	if(dynamic_cast<TComponentOwner *>(this))
	{
		// If we have a designer, and 
		// a component has been selected on the palette,
		// make like a form designer
		if((Designer()) && (Designer()->IsDropWaiting()))
		{
			uint32 buttons; 
		  
		  // Draw a tracking rectangle
		  BRect rect(pt, pt); 
		  DTView()->BeginRectTracking(rect, B_TRACK_RECT_CORNER); 
		  do 
		  { 
		  	snooze(30 * 1000); 
		    DTView()->GetMouse(&pt, &buttons); 
		  } while(buttons); 
		  DTView()->EndRectTracking(); 
		
		  rect.SetRightBottom(pt);
		  
		  // Make sure the control is at least as big as minimum size
		  if(rect.Width() < MIN_COMPONENT_SIZE)
		  	rect.right = rect.left + MIN_COMPONENT_SIZE;
		  	
		  if(rect.Height() < MIN_COMPONENT_SIZE)
		  	rect.bottom = rect.top + MIN_COMPONENT_SIZE;
		  
		 	// Get the new component
		 	TComponent *component = Designer()->CreateComponent(DTView(), rect);
		 	if(component)
		 	{
		 		// Add the child as a sub-component of this component.
		 		TComponentOwner *owner = dynamic_cast<TComponentOwner *>(this);
		 		if(owner)
		 		{
		 			owner->AddComponent(component);
		 		}
		 		
		 		printf(
		 			"%s: Created component '%s'\n"
		 		, class_name(this), component->ComponentName()
		 		);
		 		Designer()->DoneDroppingComponent();
		 	}
		}
		else // This is a parent-type control being moved, as opposed to pretending 
		 		 // it's a form designer.
		{
			goto lbl_NonParentComponent;
		}
	}
	else // this is not time to act like a parent-type control
	{
lbl_NonParentComponent: // OH NO A LABEL!	
		uint32 buttons;
		BPoint where;
		view->GetMouse(&where, &buttons);
		
		if(buttons & B_PRIMARY_MOUSE_BUTTON)
		{
			if(bounds.Contains(pt))
			{
				// If the user is dragging the far right-bottom, they're
				// resizing, otherwise they're moving
				if(
					 ((pt.x >= (bounds.right-RESIZE_OFFSET)) && (pt.y >= (bounds.bottom-RESIZE_OFFSET)))
				&& (ctVisual == ComponentType())
				)
				{
				  BRect rect(bounds.LeftTop(), pt); 
				  uint32 buttons;
				  
				  view->BeginRectTracking(rect, B_TRACK_RECT_CORNER); 
				  do 
				  { 
				  	snooze(30 * 1000); 
				    view->GetMouse(&pt, &buttons); 
				  } while(buttons); 
				  view->EndRectTracking(); 
			
			  	rect.SetRightBottom(pt);
			  	
			  	if(
			  		 (rect.Width() >= MIN_COMPONENT_SIZE) 
			  	&& (rect.Height() >= MIN_COMPONENT_SIZE)
			  	&& (!Designer()->Locked())
			  	)
			  	{
			  		ResizeComponentTo(view, rect);
			  	}
			  	
			  	if(!Designer()->Locked())
			  	{
				  	rect = view->Frame();
				  	
				  	// Inform the designer that the component was moved, 
				  	// and tell it its new location on the screen.
				  	BMessage resizeMessage(DTM_COMPONENT_RESIZE);
				  	resizeMessage.AddPointer("component", this);
				  	resizeMessage.AddRect("frame", rect);
				  	view->Window()->PostMessage(&resizeMessage, view->Parent());
					}
				}
				else
				{
					if(!Designer()->Locked())
					{
						// Start a drag-n-drop session to move this and any other components that are selected
						BMessage dragMsg(DTM_COMPONENT_MOVE);
						dragMsg.AddPointer("component", this);
						view->DragMessage(&dragMsg, bounds);
					}
				}
			}
		}
		else if(buttons & B_SECONDARY_MOUSE_BUTTON)
		{
			// Tell the designer that we want to show the context menu.
			BMessage rmMessage(DTM_COMPONENT_CONTEXTMENU);
			rmMessage.AddPoint("where", where);
			rmMessage.AddPointer("view", view);
			rmMessage.AddPointer("component", this);
	  	view->Window()->PostMessage(&rmMessage, view->Parent());
		}
	}
}

void TComponent::DesignerDrawSelection(void)
{
	if((DTView()) && (IsDesignerSelected()))
	{
		rgb_color rectColor = clSelectionRect;
		if(Designer()->Locked())
			rectColor = clLockedSelectionRect;

		BRect bounds = DTView()->Bounds();
		
		// Draw a border around this view.
		DTView()->LockLooper();
		DTView()->BeginLineArray(4);
		
		DTView()->AddLine(BPoint(0,0), BPoint(bounds.Width(), 0), rectColor);
		DTView()->AddLine(BPoint(bounds.Width(), 0), BPoint(bounds.Width(), bounds.Height()), rectColor);
		DTView()->AddLine(BPoint(bounds.Width(), bounds.Height()), BPoint(0,bounds.Height()), rectColor);
		DTView()->AddLine(BPoint(0,bounds.Height()), BPoint(0,0), rectColor);
		
		DTView()->EndLineArray();
		
		if((!Designer()->Locked()) && (ctVisual == ComponentType()))
		{
			BRect r(
				bounds.Width()-RESIZE_OFFSET, 
				bounds.Height()-RESIZE_OFFSET, 
				bounds.Width(), 
				bounds.Height()
			);
			rgb_color clr = DTView()->HighColor();
			
			DTView()->SetHighColor(rectColor);
			DTView()->FillRect(r);
			DTView()->SetHighColor(clr);
		}
		
		DTView()->UnlockLooper();
	}
}

bool TComponent::HandleComponentDesignMessage(BMessage *message)
{
	bool result = false;
	
	switch(message->what)
	{
		case DTM_COMPONENT_MOVE:
		{
			printf("%s: DTM_COMPONENTMOVE\n", class_name(this));
			
			BRect rect;
			TComponent *component = 0;
			
			if(B_OK == message->FindPointer("component", (void **)&component))
			{
				// If the component being moved is THIS control, forward the
				// DTM_COMPONENT_MOVE message on to the parent view.
				if(component == dynamic_cast<TComponent *>(this))
				{
					printf("%s selected\n", class_name(this));
					
					BView *parent = DTView()->Parent();
					if(parent)
					{
						// forward this message to the parent view
						BMessage fwd(*message);
						parent->Window()->PostMessage(&fwd, parent);
					}
					
					break;
				}
				else
				{
					// If somebody dropped a control
					if(message->WasDropped())
					{
						int32 
							oldLeft
						, oldTop
						, newLeft
						, newTop
						;
						
						// Save the component's current position
						oldLeft = (int32)component->DTView()->Frame().left;	
						oldTop = (int32)component->DTView()->Frame().top;
						
						BPoint offset;
						BPoint where = message->DropPoint(&offset);
						DTView()->ConvertFromScreen(&where);
						// Correct for the size of the control
						where.x -= offset.x;
						where.y -= offset.y;
						
						// If the control being moved is not:
						// * a child of this control
						// * this control, or
						// * the parent of this control,
						// remove it from its parent and adopt it.
						if(
							(component->DTView()->Parent() != DTView()) && 
							(component->DTView() != DTView()) &&
							(component->DTView() != DTView()->Parent())
						)
						{
							printf("%s Adding child %s from other parent\n", class_name(this), component->ComponentName());
							TComponentOwner *owner = dynamic_cast<TComponentOwner *>(this);
							if(owner)
							{
								// if the component has an owner,
								if(component->Owner())
								{
									TComponentOwner *designer = (TComponentOwner *)component->Designer();
									TComponentOwner *other = (TComponentOwner *)component->Owner();
									
									// and the owner is not the design form
									if(other != designer)
									{
										// tell the other owner to give up the component.
										component->Owner()->RemoveComponent(component);
									}
								}
								
								owner->AddComponent(component);
							}
							
							component->DTView()->Parent()->RemoveChild(component->DTView());
							DTView()->AddChild(component->DTView());
						}

						// Move the component to the mouse location.
						component->MoveComponentTo(where);
						if(component->DTView()->LockLooper())
						{
							rect = component->DTView()->Frame();
							component->DTView()->UnlockLooper();
						}
						
						// Save the component's new position
						newLeft = (int32)rect.left;
						newTop = (int32)rect.top;
						
						BRect newRect;
						Designer()->UpdateComponentPosition(component, &newRect);

						if(Designer()->SelectionCount())
						{
							for(uint32 i = 0; i < Designer()->SelectionCount(); i++)
							{
								TComponent *c = Designer()->SelectedComponentAt(i);
								if(c && c != component)
								{
		    					c->DTView()->MoveBy((newLeft-oldLeft),(newTop-oldTop));
		    					newRect = c->DTView()->Frame();
		    					Designer()->UpdateComponentPosition(c, &newRect);
									if(c->DTView()->Parent() != DTView())
									{
										printf("%s (multi-move) Adding child from other parent\n", class_name(this));
										TComponentOwner *owner = dynamic_cast<TComponentOwner *>(this);
										if(owner)
										{
											// if the component has an owner,
											if(c->Owner())
											{
												TComponentOwner *designer = (TComponentOwner *)c->Designer();
												TComponentOwner *other = (TComponentOwner *)c->Owner();
												
												// and the owner is not the design form
												if(other != designer)
												{
													// tell the other owner to give up the component.
													c->Owner()->RemoveComponent(c);
												}
											}
											
											owner->AddComponent(c);
										}
										
										c->DTView()->Parent()->RemoveChild(c->DTView());
										DTView()->AddChild(c->DTView());
									}
								}
							}
						}
					}
					else
					{
						rect = component->DTView()->Frame();
					}
				}
			
				rect = component->DTView()->Frame();
				// If this component's parent is the designer,
				// tell it about this control being moved
				if(Designer())
				{
					Designer()->ComponentMoved(
						component
					, rect.LeftTop()
					);
				}

			}
			
			result = true;
			break;
		}
		
		default:
		{
			result = false;
		}
	}
	return result;
}

void TComponent::AddPropAndEditor(
	TProperty *prop
, TPropertyEditor *editor
,	const char *setStatement
, const char *getStatement
, bool quoted
)
{
	if((prop) && (editor))
	{
		prop->SetEditor(editor);
		editor->SetProperty(prop);
		
		if(setStatement) prop->SetSetStatement(setStatement);
		if(getStatement) prop->SetGetStatement(getStatement);
		prop->SetQuoted(quoted);
	}
	AddProperty(prop);
}

void TComponent::MoveComponentTo(BView *vw, BPoint pt) 
{ 
	if(vw->LockLooper())
	{ 
		vw->MoveTo(pt); 
		vw->UnlockLooper();
	}
}

void TComponent::MoveComponentTo(BPoint pt) 
{ 
	if(m_DTView) 
	{
		if(m_DTView->LockLooper())
		{
			m_DTView->MoveTo(pt);
			m_DTView->UnlockLooper();
		}
  }
}


//
// TComponentOwner implementation
//
TComponentOwner::~TComponentOwner(void)
{
//	ClearComponents();
//	if(m_Components) 
//		delete m_Components;
}

void TComponentOwner::SetTabOrders(void)
{
	// nothing here yet
}


//
// TAppComponent implementation
//
void TAppComponent::InitPropertyList(void)
{
	TComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "ClassName", "TMyApplication")
	, new TStringProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "Signature", "application/x-vnd.MyApp")
	, new TStringProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "LaunchFlags", "B_SINGLE_LAUNCH")
	, new TLaunchFlagsProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "GenerationPath", "/boot/home/MyApp")
	, new TStringProperty(this, 0)
	);
}

void TAppComponent::InitEventList(void)
{
	TComponent::InitEventList();

	AddEvent(new TEvent("OnAboutRequested"));
	AddEvent(new TEvent("OnAppActivated", "bool active"));
	AddEvent(new TEvent("OnArgvReceived", "int32 argc, char **argv"));
	AddEvent(new TEvent("OnPulse"));
	AddEvent(new TEvent("OnQuitRequested", 0, "bool"));
	AddEvent(new TEvent("OnReadyToRun"));
	AddEvent(new TEvent("OnRefsReceived", "BMessage *message"));
}

void TAppComponent::InitPropertyValues(void)
{
	TComponent::InitPropertyValues();
	
}

void TAppComponent::InitEventValues(void)
{
	TComponent::InitEventValues();
	
}	

const char *TAppComponent::GetPropertyValue(const char *propName) const
{
	printf("TAppComponent::GetPropertyValue('%s')\n", propName);

	string s = propName;
	
	if(0 == strcmp("ClassName", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "TMyApp";
	}
	
	if(0 == strcmp("Signature", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "application/x-vnd.MyApp";
	}

	if(0 == strcmp("LaunchFlags", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "B_SINGLE_LAUNCH";
	}

	if(0 == strcmp("GenerationPath", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "./MyApp";
	}

	return TComponent::GetPropertyValue(propName);
}

// Set the property value (converting from text if nec.) for the property
// referenced by "propName"
void TAppComponent::SetPropertyValue(const char *propName, const char *value)
{
	printf("TAppComponent::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	string s = propName;
	string v = (value)? value: "";
	
	if(
		(s == "ClassName")
	||(s == "LaunchFlags")
	||(s == "GenerationPath")
	||(s == "Signature")
	)
	{
		StoreProperty(propName, value);
		return;
	}
	
	TComponent::SetPropertyValue(propName, value);
}

//
// TVisualComponent implementation
//
void TVisualComponent::OnSetConstructorParams(BString &s)
{
	TComponent::OnSetConstructorParams(s);
}

void TVisualComponent::InitPropertyList(void)
{
	TComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "Visible", "true")
	, new TBoolProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "Left", "0", 0, false)
	, new TIntProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "Top", "0", 0, false)
	, new TIntProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "Right", "0", 0, false)
	, new TIntProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "Bottom", "0", 0, false)
	, new TIntProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "Flags", "B_WILL_DRAW|B_NAVIGABLE")
	, new TViewFlagsProperty(this, 0)
	);

	const char *csViewColorSet = 
		"{rgb_color clr = $VALUE;\n"
		"$COMPONENT->SetViewColor(clr);}"
		;
/*
	const char *csLowColorSet = 
		"{rgb_color clr = $VALUE;\n"
		"$COMPONENT->SetLowColor(clr);}"
		;
*/
	const char *csHighColorSet = 
		"{rgb_color clr = $VALUE;\n"
		"$COMPONENT->SetHighColor(clr);}"
		;

	AddPropAndEditor(
		new TProperty(this, "ViewColor", "{216,216,216,255}")
	, new TViewColorProperty(this, 0)
	, csViewColorSet, 0, false
	);

	AddPropAndEditor(
		new TProperty(this, "HighColor", "{0,0,0,0}")
	, new TViewColorProperty(this, 0)
	, csHighColorSet, 0, false
	);

/*
	AddPropAndEditor(
		new TProperty(this, "LowColor", "{255,255,255,255}")
	, new TViewColorProperty(this, 0)
	, csLowColorSet, 0, false
	);
*/

	AddPropAndEditor(
		new TProperty(this, "FontSize", "10")
	, new TFontSizeProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "Font", "be_plain_font")
	, new TFontProperty(this, 0)
	);
}

void TVisualComponent::InitEventList(void)
{
	TComponent::InitEventList();
	
	AddEvent(new TEvent("OnAllAttached"));
	AddEvent(new TEvent("OnAttachedToWindow"));
	AddEvent(new TEvent("OnAllDetached"));
	AddEvent(new TEvent("OnDraw", "BRect updateRect"));
	AddEvent(new TEvent("OnFrameMoved", "BPoint parentPoint"));
	AddEvent(new TEvent("OnFrameResized", "float width, float height"));
	AddEvent(new TEvent("OnKeyDown", "const char *bytes, int32 numBytes"));
	AddEvent(new TEvent("OnKeyUp", "const char *bytes, int32 numBytes"));
	AddEvent(new TEvent("OnGotFocus"));
	AddEvent(new TEvent("OnLostFocus"));
	AddEvent(new TEvent("OnMouseUp", "BPoint point"));
	AddEvent(new TEvent("OnMouseDown", "BPoint point"));
	AddEvent(new TEvent("OnMouseMoved", "BPoint point, uint32 transit, const BMessage *message"));
}

void TVisualComponent::InitPropertyValues(void)
{
	TComponent::InitPropertyValues();
	
}

void TVisualComponent::InitEventValues(void)
{
	TComponent::InitEventValues();
	
}

const char *TVisualComponent::GetPropertyValue(const char *propName) const
{
	printf("TVisualComponent::GetPropertyValue('%s')\n", propName);

	string s = propName;
	char buf[80];
	BRect rect;
	
	if(s == "Visible")
	{
		bool b;
		if(B_OK == PropertyStore()->FindBool(propName, &b))
		{
			return (b ? "true" : "false");
		}
		else return "true";
	}
	
	if(0 == strcmp("ViewColor", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "{216,216,216,255}";
	}
	
	if(0 == strcmp("HighColor", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "{0,0,0,0}";
	}

	if(0 == strcmp("LowColor", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "{255,255,255,255}";
	}

	if(s == "Left")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.left);
			return buf;
		}
	}

	if(s == "Top")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.top);
			return buf;
		}
	}
	
	if(s == "Right")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.right);
			return buf;
		}
	}
	
	if(s == "Bottom")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.bottom);
			return buf;
		}
	}
	
	return TComponent::GetPropertyValue(propName);
}

void TVisualComponent::SetPropertyValue(const char *propName, const char *value)
{
	printf("TVisualComponent::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	BRect r;
	BPoint pt;
	string s = propName;
	string v = (value)? value: "";
	bool b = false;
	
	if(s == "Visible")
	{
		StoreProperty(propName, value);
		return;
	}

	if(s == "Left")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			pt = DTView()->Frame().LeftTop();
			pt.x = atol(value);
			DTView()->MoveTo(pt);
			DTView()->UnlockLooper();	
			b = true;
		}
	}

	if(s == "Top")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			pt = DTView()->Frame().LeftTop();
			pt.y = atol(value);
			DTView()->MoveTo(pt);
			DTView()->UnlockLooper();	
			b = true;
		}
	}

	if(s == "Right")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			r = DTView()->Frame();
			r.right = atol(value);
			DTView()->ResizeTo(r.Width(), r.Height());
			DTView()->UnlockLooper();	
			b = true;
		}
	}
	
	if(s == "Bottom")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			r = DTView()->Frame();
			r.bottom = atol(value);
			DTView()->ResizeTo(r.Width(), r.Height());
			DTView()->UnlockLooper();	
			b = true;
		}
	}
	
	if(s == "Font")
	{
		if(DTView() && DTView()->LockLooper())
		{
			if(v == "be_plain_font") DTView()->SetFont(be_plain_font);
			if(v == "be_bold_font")  DTView()->SetFont(be_bold_font);
			if(v == "be_fixed_font") DTView()->SetFont(be_fixed_font);
			
			DTView()->UnlockLooper();
			b = true;
		}
	}
	
	if(s == "FontSize")
	{
		if(DTView() && DTView()->LockLooper())
		{
			int32 lFontSize = atol(v.c_str());
			BFont font;
			
			DTView()->GetFont(&font);
			font.SetSize(lFontSize);
			DTView()->SetFont(&font, B_FONT_SIZE);
			
			DTView()->UnlockLooper();
			b = true;
		}
	}
	
	if(s == "ViewColor")
	{
		if(DTView()->LockLooper())
		{
			rgb_color clr;
			StrToRGBColor(value, &clr);
			DTView()->SetViewColor(clr);
			DTView()->UnlockLooper();
			StoreProperty(propName, value);
			b = true;
		}
	}

	if(s == "HighColor")
	{
		if(DTView()->LockLooper())
		{
			rgb_color clr;
			StrToRGBColor(value, &clr);
			DTView()->SetHighColor(clr);
			DTView()->UnlockLooper();
			StoreProperty(propName, value);
			b = true;
		}
	}

	if(s == "LowColor")
	{
		if(DTView()->LockLooper())
		{
			rgb_color clr;
			StrToRGBColor(value, &clr);
			DTView()->SetLowColor(clr);
			DTView()->UnlockLooper();
			StoreProperty(propName, value);
			b = true;
		}
	}

	if(b)
	{
		if(DTView() && DTView()->LockLooper())
		{
			DTView()->Invalidate();
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	TComponent::SetPropertyValue(propName, value);
}

//
// TNonVisualComponent implementation
//
void TNonVisualComponent::OnSetConstructorParams(BString &s)
{
	TComponent::OnSetConstructorParams(s);
}

void TNonVisualComponent::InitPropertyList(void)
{
	TComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "Left", "0", 0, false)
	, new TIntProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "Top", "0", 0, false)
	, new TIntProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "Right", "0", 0, false)
	, new TIntProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "Bottom", "0", 0, false)
	, new TIntProperty(this, 0)
	);
	
	FindProperty("Left")->SetGenerated(false);
	FindProperty("Top")->SetGenerated(false);
	FindProperty("Right")->SetGenerated(false);	
	FindProperty("Bottom")->SetGenerated(false);	
}

void TNonVisualComponent::InitEventList(void)
{
	TComponent::InitEventList();
}

void TNonVisualComponent::InitPropertyValues(void)
{
	TComponent::InitPropertyValues();
}

void TNonVisualComponent::InitEventValues(void)
{
	TComponent::InitEventValues();
}

const char *TNonVisualComponent::GetPropertyValue(const char *propName) const
{
	printf("TNonVisualComponent::GetPropertyValue('%s')\n", propName);

	string s = propName;
	char buf[80];
	BRect rect;
	
	if(s == "Left")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.left);
			return buf;
		}
	}

	if(s == "Top")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.top);
			return buf;
		}
	}
	
	if(s == "Right")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.right);
			return buf;
		}
	}
	
	if(s == "Bottom")
	{
		if(B_OK == PropertyStore()->FindRect("Frame", &rect))
		{
			sprintf(buf, "%ld", (uint32)rect.bottom);
			return buf;
		}
	}
	
	return TComponent::GetPropertyValue(propName);
}

void TNonVisualComponent::SetPropertyValue(const char *propName, const char *value)
{
	printf("TNonVisualComponent::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	BRect r;
	BPoint pt;
	string s = propName;
	string v = (value)? value: "";
	bool b = false;
	
	if(s == "Left")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			pt = DTView()->Frame().LeftTop();
			pt.x = atol(value);
			DTView()->MoveTo(pt);
			DTView()->UnlockLooper();	
			b = true;
		}
	}

	if(s == "Top")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			pt = DTView()->Frame().LeftTop();
			pt.y = atol(value);
			DTView()->MoveTo(pt);
			DTView()->UnlockLooper();	
			b = true;
		}
	}

	if(s == "Right")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			r = DTView()->Frame();
			r.right = atol(value);
			DTView()->ResizeTo(r.Width(), r.Height());
			DTView()->UnlockLooper();	
			b = true;
		}
	}
	
	if(s == "Bottom")
	{
		if(DTView() && DTView()->LockLooper())	
		{
			r = DTView()->Frame();
			r.bottom = atol(value);
			DTView()->ResizeTo(r.Width(), r.Height());
			DTView()->UnlockLooper();	
			b = true;
		}
	}
	
	if(b)
	{
		if(DTView() && DTView()->LockLooper())
		{
			DTView()->Invalidate();
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	TComponent::SetPropertyValue(propName, value);
}

//
// TViewComponent implementation
//
// Load the list of properties for this component
void TViewComponent::OnSetConstructorParams(BString &s)
{
	TVisualComponent::OnSetConstructorParams(s);
}

void TViewComponent::InitPropertyList(void)
{
	TVisualComponent::InitPropertyList();
}

void TViewComponent::SetPropertyValue(const char *propName, const char *value)
{
	TVisualComponent::SetPropertyValue(propName, value);	
}

const char *TViewComponent::GetPropertyValue(const char *propName) const
{
	return TVisualComponent::GetPropertyValue(propName);	
}


// Load the list of events for this component
void TViewComponent::InitEventList(void)
{
	TVisualComponent::InitEventList();
}

// Initialize design-time property values
void TViewComponent::InitPropertyValues(void)
{
	TVisualComponent::InitPropertyValues();
}

// Initialize design-time event values
void TViewComponent::InitEventValues(void)
{
	TVisualComponent::InitEventValues();
}

//
// TFormComponent implementation
//
void TFormComponent::InitPropertyList(void)
{
	TVisualComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "Title", "Form")
	, new TStringProperty(this, 0)
	, 0, 0, true
	);

	AddPropAndEditor(
		new TProperty(this, "WindowLook", "B_TITLED_WINDOW_LOOK")
	, new TWindowLookProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "WindowFeel", "B_NORMAL_WINDOW_FEEL")
	, new TWindowFeelProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "WindowFlags", "")
	, new TWindowFlagsProperty(this, 0)
	);
}

void TFormComponent::InitEventList(void)
{
	TVisualComponent::InitEventList();

	AddEvent(new TEvent("OnMenusBeginning"));
	AddEvent(new TEvent("OnMenusEnded"));
	AddEvent(new TEvent("OnOpenViewTransaction"));
	AddEvent(new TEvent("OnCommitViewTransaction"));
	AddEvent(new TEvent("OnQuit"));
	AddEvent(new TEvent("OnScreenChanged", "BRect frame, color_space mode"));
	AddEvent(new TEvent("OnShow"));
	AddEvent(new TEvent("OnHide"));
	AddEvent(new TEvent("OnMinimize", "bool minimize"));
	AddEvent(new TEvent("OnZoom", "BPoint origin, float width, float height"));
	AddEvent(new TEvent("OnWindowActivated", "bool active"));
	AddEvent(new TEvent("OnWorkspaceActivated", "int32 workspace, bool active"));
	AddEvent(new TEvent("OnWorkspacesChanged", "uint32 oldWorkspaces, uint32 newWorkspaces"));
	AddEvent(new TEvent("OnQuitRequested", NULL, "bool"));
}

void TFormComponent::InitPropertyValues(void)
{
	TVisualComponent::InitPropertyValues();
	
}

void TFormComponent::InitEventValues(void)
{
	TVisualComponent::InitEventValues();
	
}

const char *TFormComponent::GetPropertyValue(const char *propName) const
{
	printf("TFormComponent::GetPropertyValue('%s')\n", propName);

	string s = propName;
	
	if(s == "WindowFlags")
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "";
	}
	
	if(s == "WindowFeel")
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "B_NORMAL_WINDOW_FEEL";
	}
	
	if(s == "WindowLook")
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "B_TITLED_WINDOW_LOOK";
	}
	
	if(s == "Title")
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "Form";
	}

	return TVisualComponent::GetPropertyValue(propName);
}

void TFormComponent::SetPropertyValue(const char *propName, const char *value)
{
	printf("TFormComponent::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	string s = propName;
	string v = (value) ? value : "";

	if(s == "WindowLook")
	{
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "WindowFlags")
	{
		StoreProperty(propName, value);
		return;
	}	
	
	// Only set window feel at runtime
	if(s == "WindowFeel")
	{
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "Title")
	{
		StoreProperty(propName, value);
		return;
	}
	
	TVisualComponent::SetPropertyValue(propName, value);
}

//
// TControlComponent implementation
//
void TControlComponent::InitPropertyList(void)
{
	TViewComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "Enabled", "true")
	, new TBoolProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "Label", "")
	, new TStringProperty(this, 0)
	, 0, 0, true
	);
	
	AddPropAndEditor(
		new TProperty(this, "ResizingMode", "B_FOLLOW_LEFT|B_FOLLOW_TOP")
	, new TResizingModeProperty(this, 0)
	);	
}

void TControlComponent::InitEventList(void)
{
	TViewComponent::InitEventList();
}

void TControlComponent::InitPropertyValues(void)
{
	TViewComponent::InitPropertyValues();
}

void TControlComponent::InitEventValues(void)
{
	TViewComponent::InitEventValues();
}

const char *TControlComponent::GetPropertyValue(const char *propName) const
{
	printf("TControlComponent::GetPropertyValue('%s')\n", propName);

	if(0 == strcmp("ResizingMode", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "B_FOLLOW_NONE";
	}
	
	if(0 == strcmp("Enabled", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "true";
	}
	
	if(0 == strcmp("Flags", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
			return "B_WILL_DRAW|B_NAVIGABLE";
	}
	
	if(0 == strcmp("Label", propName))
	{
		const char *buf = 0;
		if(B_OK == PropertyStore()->FindString(propName, &buf))
			return buf;
		else
		{
			BControl *control = dynamic_cast<BControl *>((TControlComponent *)this);
			if(control)
				return control->Label();
			else
				return ComponentName();
		}
	}
	
	return TViewComponent::GetPropertyValue(propName);
}

void TControlComponent::SetPropertyValue(const char *propName, const char *value)
{
	printf("TControlComponent::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	if(
		 (0 == strcmp("ResizingMode", propName))
	|| (0 == strcmp("Flags", propName))
	)
	{
		StoreProperty(propName, value);
		return;
	}
	
	if(0 == strcmp("Label", propName))
	{
		if(DTView() && DTView()->LockLooper())
		{
			BControl *control = dynamic_cast<BControl *>(this);
			if(control)
			{
				control->SetLabel(value);
			}
			DTView()->UnlockLooper();
		}
		else
			printf("Warning: %s::DTView() not set\n", class_name(this));
			
		StoreProperty(propName, value);
		return;
	}
	
	if(0 == strcmp("Enabled", propName))
	{
		if(DTView() && DTView()->LockLooper())
		{
			BControl *control = dynamic_cast<BControl *>(this);
			if(control)
			{
				control->SetEnabled((0 == strcmp(value, "true")));
			}
			DTView()->Invalidate();
			DTView()->UnlockLooper();
		}
		else
			printf("Warning: %s::DTView() not set\n", class_name(this));
	}

	if(0 == strcmp("ViewColor", propName))
	{
		rgb_color clr;
		StrToRGBColor(value, &clr);
		if(DTView() && DTView()->LockLooper())
		{
			DTView()->SetViewColor(clr);
			DTView()->Invalidate();
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	TViewComponent::SetPropertyValue(propName, value);
}

//
// Property Editors
//
bool isnumeric(const char *text)
{
	const char *numbers = "0123456789";
	if(text)
	{
		for(uint32 i = 0; i < strlen(text); i++)
		{
			if(!strchr(numbers, text[i]))
			{
				return false;
			}
		}
		return true;
	}
	else
		return false;
}

rgb_color StrToRGBColor(const char *s, rgb_color *pclr)
{
	if(s && pclr)
	{
		char buf[T_PROP_VALUE_LENGTH] = "";
		char *pbuf = buf;
		char *p = (char *)s;
		int i = 0;
		
		// strip off leading crap
		while(' ' == *p) p++;
		while('{' == *p) p++;
		
		strcpy(buf, p);
		p = buf + (strlen(buf)-1);
		while(' ' == *p) *p-- = 0;
		while('}' == *p) *p-- = 0;
		
		p = strtok_r(buf, ",", &pbuf);
		while(p)
		{
			switch(i++)
			{
				case 0: pclr->red 	= atol(p); break;
				case 1: pclr->green = atol(p); break;
				case 2: pclr->blue	= atol(p); break;
				case 3: pclr->alpha = atol(p); break;
			}
			
			p = strtok_r(0, ",", &pbuf);
		}
	}
	return *pclr;
}

const char *RGBColorToStr(rgb_color clr, char *buf)
{
	sprintf(buf, "{%d,%d,%d,%d}", clr.red, clr.green, clr.blue, clr.alpha);
	return buf;
}

//
// RGB Color editor
//
class RGBColorWin: public TPropertyModalEdit {
	enum {
		CLR_CHANGE = 'clrC'
	};
public:
	RGBColorWin(TPropertyEditor *editor, char *value, uint32 maxlen, TComponent **selected, uint32 count)
	: TPropertyModalEdit(editor, value, maxlen, selected, count)
	{
		if(value) strncpy(m_Value, value, sizeof(m_Value));
		
		InitWindow();
		Show();
		InitControls();
	}
	
	virtual void InitWindow()
	{
		if(LockLooper())
		{
			int32 iTop, iOffset;
			iTop = 10;
			iOffset = 15;
			BRect r;
			r.Set(iTop,10,150,iTop+10);
			
			BBox *box = new BBox(Bounds(), NULL, B_FOLLOW_ALL_SIDES);
			AddChild(box);

			// Add the color control
			box->AddChild(
				ccColors = new BColorControl(
					BPoint(10,10)
				, B_CELLS_32x8 
				, 8
				, "ccColors"
				, new BMessage(CLR_CHANGE)
				, false
				)
			);
			
			ResizeTo(ccColors->Frame().Width() + 30, ccColors->Frame().Height() + 50);
			
			// create OK and cancel buttons
			BButton *btnOK;
			r = Bounds();
			r.top = r.bottom - 30; r.bottom -= 10;
			r.left = 15; r.right = 65;
			box->AddChild(btnOK = new BButton(r, "btnOK", "OK", new BMessage(PME_BTNAPPLY)));
			btnOK->MakeDefault(true);
			
			r.left += 60; r.right += 60;
			box->AddChild(new BButton(r, "btnCancel", "Cancel", new BMessage(PME_BTNCANCEL)));
			
			r.left += 80; r.right += 80;
			box->AddChild(boxTest = new BBox(r));
			boxTest->SetBorder(B_PLAIN_BORDER);
				
			UnlockLooper();
		}
	}
	
	virtual void InitControls()
	{
		// initialize control values to represent view flags
		if(LockLooper())
		{
			// translate the text into an rgb_color, and set the color controls value.
			printf("rgb_color = %s\n", m_Value);
			rgb_color clr;
			
			StrToRGBColor(m_Value, &clr);
			
			ccColors->SetValue(clr);
			boxTest->SetViewColor(clr);
			
			UnlockLooper();
		}
	}
	
	virtual void OnApply(void)
	{
		// Apply/OK/Save button pressed. Convert view flags into 
		// string representation, and copy to m_Value.
		string s = "";
		char buf[T_PROP_VALUE_LENGTH] = "";
		
		// translate the rgb_color of the color control into a text string.		
		strcpy(m_Value, RGBColorToStr(ccColors->ValueAsColor(), buf));
		
		printf("%s\n", m_Value);
	}
	
	virtual bool OnMessage(BMessage *msg)
	{
		switch(msg->what)
		{
			case CLR_CHANGE:
			{
				boxTest->SetViewColor(ccColors->ValueAsColor());
				boxTest->Invalidate();
				return true;
			}
			
			default:
				return false;
		}
	}
	
	BColorControl 
		*ccColors;
		
	BBox
		*boxTest;
	
};

// TRGBColorProperty implementation
bool TRGBColorProperty::ModalEdit(TComponent **selected, uint32 count)
{
	// create a modal dialog with A BColorControl on it to select the color.
	char buf[T_PROP_VALUE_LENGTH] = "";
	strncpy(buf, Property()->Value(), sizeof(buf));
	new RGBColorWin(this, buf, sizeof(buf), selected, count);
	return true;
}

// TStringListProperty implementation
void TStringListProperty::GetValueList(TStringList *list)
{
	list->MakeEmpty();
}

void TStringListProperty::SetValueFromList(const TStringList *list)
{
	// The most common use of a list of values from this property is to 
	// delimit them with "|" characters, i.e., for resize-mode flags, etc.
	BString value = "";
	
	for(int32 i = 0; i < list->CountItems(); i++)
	{
		value << "|" << (const char *)list->ItemAt(i);
	}
	value.RemoveFirst("|");
	SetValue(value.String());
}

// TEnumProperty implementation
void TEnumProperty::GetValuePickList(TStringList *list)
{
	list->MakeEmpty();
}

const char *TEnumProperty::GetDisplayValue() const
{
	return GetValue();
}

// TComponentProperty implementation
void TComponentProperty::SetComponentRef(TComponent *comp)
{
	m_ComponentRef = comp;
	if(comp)
	{
		SetValue(comp->ComponentName());
		comp->AddPropertyListener(this);
		
		// ALWAYS make sure that this property's component is later in the 
		// tab order than the referenced component!
		if(Component()->TabOrder() < comp->TabOrder())
		{
			int t = Component()->TabOrder();
			int x = comp->TabOrder();
			comp->SetTabOrder(t);
			Component()->SetTabOrder(x);
		}
	}
	else
	{
		SetValue("NULL");
	}
}

TComponent *TComponentProperty::ComponentRef(void) const
{
	return m_ComponentRef;
}

void TComponentProperty::RemoveComponentRef(void)
{
	if(m_ComponentRef)
	{
		m_ComponentRef->RemovePropertyListener(this);
	}
	m_ComponentRef = NULL;
}

void TComponentProperty::GetValuePickList(TStringList *list)
{
	list->MakeEmpty();
	list->Add("(None)");
	
	if(Component())
	{
		for(int32 i = 0; i < Component()->Designer()->CountComponents(); i++)
		{
			TComponent *c = Component()->Designer()->ComponentAt(i);
			if(c != Component())
			{
				list->Add(c->ComponentName());
			}
		}
	}
	else
		printf("Oops! No component!\n");
}

const char *TComponentProperty::GetValue(void) const
{
	if(NULL != ComponentRef())
	{
		Property()->SetValue(ComponentRef()->ComponentName());
		return ComponentRef()->ComponentName();
	}
	else
	{
		Property()->SetValue(NULL);
		return NULL;
	}
}

const char *TComponentProperty::GetDisplayValue(void) const
{
	if(NULL != ComponentRef())
	{	
		Property()->SetValue(ComponentRef()->ComponentName());
		return ComponentRef()->ComponentName();
	}
	else
	{
		Property()->SetValue(NULL);
		return "(None)";
	}
}

void TComponentProperty::PropListenerNotify(TComponent *c, const char *name, const char *oldval, const char *newval)	
{
	if(ComponentRef() == c)
	{
		if(!strcmp(name, "Name"))
		{
			SetValue(newval);
		}
	}
}

void TWindowLookProperty::GetValuePickList(TStringList *list)
{
	list->MakeEmpty();
	list->Add("B_BORDERED_WINDOW_LOOK");
	list->Add("B_TITLED_WINDOW_LOOK");
	list->Add("B_DOCUMENT_WINDOW_LOOK");
	list->Add("B_MODAL_WINDOW_LOOK");
	list->Add("B_FLOATING_WINDOW_LOOK");
}

void TWindowFeelProperty::GetValuePickList(TStringList *list)
{
	list->MakeEmpty();
	list->Add("B_NORMAL_WINDOW_FEEL");
	list->Add("B_MODAL_SUBSET_WINDOW_FEEL");
	list->Add("B_MODAL_APP_WINDOW_FEEL");
	list->Add("B_MODAL_ALL_WINDOW_FEEL");
	list->Add("B_FLOATING_SUBSET_WINDOW_FEEL");
	list->Add("B_FLOATING_APP_WINDOW_FEEL");
	list->Add("B_FLOATING_ALL_WINDOW_FEEL");
}

enum WFWIDs {
	WFW_BTNOK
, WFW_BTNCANCEL	
};

class WindowFlagsWin: public BWindow {
public:
	WindowFlagsWin(TPropertyEditor *editor, char *value, uint32 maxlen)
	: BWindow(BRect(0,0,190,190), "Window Flags", B_MODAL_WINDOW, B_NOT_ZOOMABLE)
	, OKClicked(false)
	, m_MaxLen(maxlen)
	, m_Editor(editor)
	{
		strcpy(m_Value, value);
		
		BPoint pt(100,100);
		MoveTo(pt);
		
		SetFlags(Flags() | B_NOT_RESIZABLE);
		SetFeel(B_MODAL_APP_WINDOW_FEEL);

		int32 iTop, iOffset;
		iTop = 10;
		iOffset = 15;
		BRect r;
		r.Set(iTop,10,150,iTop+10);
		
		BBox *boxPanel = new BBox(Bounds(), NULL, B_FOLLOW_ALL_SIDES);
		AddChild(boxPanel);
		
		// Add the check boxes
		boxPanel->AddChild(chkNotMoveable = new BCheckBox(r,"","Not movable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotHResizeable = new BCheckBox(r,"","Not H. resizable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotVResizeable = new BCheckBox(r,"","Not V. resizable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotResizeable = new BCheckBox(r,"","Not resizable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotCloseable = new BCheckBox(r,"","Not closable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotZoomable = new BCheckBox(r,"","Not zoomable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotMinimizeable = new BCheckBox(r,"","Not minimizable",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkAcceptsFirstClick = new BCheckBox(r,"","Accepts first click",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkAvoidsFront = new BCheckBox(r,"","Avoids front",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkAvoidsFocus = new BCheckBox(r,"","Avoids focus",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkOutlineResize = new BCheckBox(r,"","Outline resize",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNoWSActivate = new BCheckBox(r,"","No workspace activation",0));
		r.top += iOffset; r.bottom += iOffset;
		boxPanel->AddChild(chkNotAnchored = new BCheckBox(r,"","Not anchored on activate",0));
		
		BButton *btnOK;
		r.top += 24; r.bottom += 34;
		r.left = 40; r.right = 90;
		boxPanel->AddChild(btnOK = new BButton(r, "btnOK", "OK", new BMessage(WFW_BTNOK)));
		btnOK->MakeDefault(true);
		
		r.left += 65; r.right += 65;
		boxPanel->AddChild(new BButton(r, "btnCancel", "Cancel", new BMessage(WFW_BTNCANCEL)));
		
		ResizeTo(Bounds().Width(), r.bottom + 10);
		Show();
	}
	
	virtual void Show(void)
	{
		// Set the check boxes for the passed-in flags
		chkNotMoveable->SetValue((strstr(m_Value, "B_NOT_MOVABLE")) ? 1 : 0);
	  chkNotHResizeable->SetValue((strstr(m_Value, "B_NOT_H_RESIZABLE")) ? 1 : 0);
	  chkNotVResizeable->SetValue((strstr(m_Value, "B_NOT_V_RESIZABLE")) ? 1 : 0);
	  chkNotResizeable->SetValue((strstr(m_Value, "B_NOT_RESIZABLE")) ? 1 : 0);
	  chkNotCloseable->SetValue((strstr(m_Value, "B_NOT_CLOSABLE")) ? 1 : 0);
	  chkNotZoomable->SetValue((strstr(m_Value, "B_NOT_ZOOMABLE")) ? 1 : 0);
	  chkNotMinimizeable->SetValue((strstr(m_Value, "B_NOT_MINIMIZABLE")) ? 1 : 0);
	  chkAcceptsFirstClick->SetValue((strstr(m_Value, "B_WILL_ACCEPT_FIRST_CLICK")) ? 1 : 0);
	  chkAvoidsFront->SetValue((strstr(m_Value, "B_AVOID_FRONT")) ? 1 : 0);
	  chkAvoidsFocus->SetValue((strstr(m_Value, "B_AVOID_FOCUS")) ? 1 : 0);
	  chkOutlineResize->SetValue((strstr(m_Value, "B_AVOID_FOCUS")) ? 1 : 0);
	  chkNoWSActivate->SetValue((strstr(m_Value, "B_NO_WORKSPACE_ACTIVATION")) ? 1 : 0);
	  chkNotAnchored->SetValue((strstr(m_Value, "B_NOT_ANCHORED_ON_ACTIVATE")) ? 1 : 0);

		BWindow::Show();
	}
	
	uint32 WindowFlags(void)
	{
		uint32 result = 0;
		
		if(B_CONTROL_ON == chkNotMoveable->Value())
			result |= B_NOT_MOVABLE;
			
		if(B_CONTROL_ON == chkNotHResizeable->Value())
			result |= B_NOT_H_RESIZABLE;

		if(B_CONTROL_ON == chkNotVResizeable->Value())
			result |= B_NOT_V_RESIZABLE;
			
		if(B_CONTROL_ON == chkNotResizeable->Value())
			result |= B_NOT_RESIZABLE;

		if(B_CONTROL_ON == chkNotCloseable->Value())
			result |= B_NOT_CLOSABLE;

		if(B_CONTROL_ON == chkNotZoomable->Value())
			result |= B_NOT_ZOOMABLE;

		if(B_CONTROL_ON == chkNotMinimizeable->Value())
			result |= B_NOT_MINIMIZABLE;

		if(B_CONTROL_ON == chkAcceptsFirstClick->Value())
			result |= B_WILL_ACCEPT_FIRST_CLICK;

		if(B_CONTROL_ON == chkAvoidsFront->Value())
			result |= B_AVOID_FRONT;

		if(B_CONTROL_ON == chkAvoidsFocus->Value())
			result |= B_AVOID_FOCUS;

		if(B_CONTROL_ON == chkOutlineResize->Value())
			result |= B_OUTLINE_RESIZE;

		if(B_CONTROL_ON == chkNoWSActivate->Value())
			result |= B_NO_WORKSPACE_ACTIVATION;

		if(B_CONTROL_ON == chkNotAnchored->Value())
			result |= B_NOT_ANCHORED_ON_ACTIVATE;
			
		return result;
	}
	
	virtual void MessageReceived(BMessage *msg)
	{
		switch(msg->what)
		{
			case WFW_BTNOK:
			{
				string s = "";
				char buf[T_PROP_VALUE_LENGTH];
				uint32 flags = WindowFlags();
				
				if(flags & B_NOT_MOVABLE) 							s += "|B_NOT_MOVABLE";
			  if(flags & B_NOT_H_RESIZABLE)						s += "|B_NOT_H_RESIZABLE";
			  if(flags & B_NOT_V_RESIZABLE)						s += "|B_NOT_V_RESIZABLE";
			  if(flags & B_NOT_RESIZABLE)							s += "|B_NOT_RESIZABLE";
			  if(flags & B_NOT_CLOSABLE)							s += "|B_NOT_CLOSABLE";
			  if(flags & B_NOT_ZOOMABLE)							s += "|B_NOT_ZOOMABLE";
			  if(flags & B_NOT_MINIMIZABLE)						s += "|B_NOT_MINIMIZABLE";
			  if(flags & B_WILL_ACCEPT_FIRST_CLICK)		s += "|B_WILL_ACCEPT_FIRST_CLICK";
			  if(flags & B_AVOID_FRONT)								s += "|B_AVOID_FRONT";
			  if(flags & B_AVOID_FOCUS)								s += "|B_AVOID_FOCUS";
			  if(flags & B_NO_WORKSPACE_ACTIVATION)		s += "|B_NO_WORKSPACE_ACTIVATION";
			  if(flags & B_NOT_ANCHORED_ON_ACTIVATE)	s += "|B_NOT_ANCHORED_ON_ACTIVATE";
			  
			  memset(buf, 0, sizeof(buf));
			  strncpy(buf, s.c_str(), s.length());
			  if(buf[strlen(buf)-1] == '|')
			  	buf[strlen(buf)-1] = 0;
			  	
			  if(buf[0] == '|')
			  {
					int32 ilen = strlen(buf);
			  	char *tmp = new char[ilen+1];
			  	strncpy(tmp, &buf[1], ilen-1);
			  	memset(buf, 0, sizeof(buf));
			  	strcpy(buf, tmp);
			  	buf[ilen-1] = 0;
			  	delete [] tmp;
			  }
			  
			  snooze(30000);
			  printf("%s\n", buf);
			  snooze(30000);
						
				m_Editor->Property()->SetValue(buf);
				m_Editor->Component()->SetPropertyValue(m_Editor->Property()->Name(), buf);
				// FALL THROUGH
			}
			
			case WFW_BTNCANCEL:
			{
				Quit();
				break;
			}
			
			default:
			{
				BWindow::MessageReceived(msg);
			}
		}
	}
	
	BCheckBox
		*chkNotMoveable
	, *chkNotHResizeable
	, *chkNotVResizeable
	, *chkNotResizeable
	, *chkNotCloseable
	, *chkNotZoomable
	, *chkNotMinimizeable
	, *chkAcceptsFirstClick
	, *chkAvoidsFront
	, *chkAvoidsFocus
	, *chkOutlineResize
	, *chkNoWSActivate
	, *chkNotAnchored;
		
	bool OKClicked;
	uint32 m_MaxLen;
	char m_Value[T_PROP_VALUE_LENGTH];
	TPropertyEditor *m_Editor;
};

bool WFWModalEdit(uint32 *pflags, char *value, uint32 maxlen)
{
	return false;
}

bool TWindowFlagsProperty::ModalEdit(TComponent **, uint32) // unused params
{
	bool result = true;
	char buf[T_PROP_VALUE_LENGTH] = "";
	strcpy(buf, Property()->Value());
	
	new WindowFlagsWin(this, buf, sizeof(buf));
	
	return result;
}

// ResizingMode property
void TResizingModeProperty::GetValueList(TStringList *list)
{
	list->MakeEmpty();
	list->Add("B_FOLLOW_LEFT");
	list->Add("B_FOLLOW_RIGHT");
	list->Add("B_FOLLOW_LEFT_RIGHT");
	list->Add("B_FOLLOW_H_CENTER");
	list->Add("B_FOLLOW_TOP");
	list->Add("B_FOLLOW_BOTTOM");
	list->Add("B_FOLLOW_TOP_BOTTOM");
	list->Add("B_FOLLOW_V_CENTER");
	list->Add("B_FOLLOW_ALL_SIDES");
	list->Add("B_FOLLOW_NONE");
}

void TResizingModeProperty::SetValueFromList(const TStringList *list)
{
	TStringListProperty::SetValueFromList(list);
}

bool TResizingModeProperty::ListItemIsSelected(const char *item)
{
	BString test(GetValue());
	return(B_ERROR != test.FindFirst(item));
}


//
// ViewFlagsWin class: for modal edits of the ViewFlags property
//
class ViewFlagsWin: public TPropertyModalEdit {
public:
	ViewFlagsWin(TPropertyEditor *editor, char *value, uint32 maxlen, TComponent **selected, uint32 count)
	: TPropertyModalEdit(editor, value, maxlen, selected, count)
	{
		if(value) strncpy(m_Value, value, sizeof(m_Value));
		
		InitWindow();
		Show();
		InitControls();
	}
	
	virtual void InitWindow()
	{
		if(LockLooper())
		{
			int32 iTop, iOffset;
			iTop = 10;
			iOffset = 15;
			BRect r;
			r.Set(iTop,10,150,iTop+10);
			
			BBox *box = new BBox(Bounds(), NULL, B_FOLLOW_ALL_SIDES);
			AddChild(box);
			
			// Add the check boxes
			box->AddChild(chkWillDraw = new BCheckBox(r,"","Will Draw",0));
			r.top += iOffset; r.bottom += iOffset;
			box->AddChild(chkPulseNeeded = new BCheckBox(r,"","Pulse Needed",0));
			r.top += iOffset; r.bottom += iOffset;
			box->AddChild(chkFrameEvents = new BCheckBox(r,"","Frame events",0));
			r.top += iOffset; r.bottom += iOffset;
			box->AddChild(chkFullUpdate = new BCheckBox(r,"","Full Update on resize",0));
			r.top += iOffset; r.bottom += iOffset;
			box->AddChild(chkNavigable = new BCheckBox(r,"","Navigable",0));
			r.top += iOffset; r.bottom += iOffset;
			box->AddChild(chkNavJump = new BCheckBox(r,"","Navigable Jump",0));
			r.top += iOffset; r.bottom += iOffset;
			box->AddChild(chkSubPixelPrecise = new BCheckBox(r,"","Sub-pixel precise",0));
			
			// create OK and cancel buttons
			BButton *btnOK;
			r.top += 20; r.bottom += 30;
			r.left = 40; r.right = 90;
			box->AddChild(btnOK = new BButton(r, "btnOK", "OK", new BMessage(PME_BTNAPPLY)));
			btnOK->MakeDefault(true);
			
			r.left += 65; r.right += 65;
			box->AddChild(new BButton(r, "btnCancel", "Cancel", new BMessage(PME_BTNCANCEL)));
			
			ResizeTo(Bounds().Width(), r.bottom + 10);
			
			UnlockLooper();
		}
	}
	
	virtual void InitControls()
	{
		// initialize control values to represent view flags
		if(LockLooper())
		{
			chkWillDraw->SetValue(strstr(m_Value, "B_WILL_DRAW") ? 1 : 0);
			chkPulseNeeded->SetValue(strstr(m_Value, "B_PULSE_NEEDED") ? 1 : 0);
			chkFrameEvents->SetValue(strstr(m_Value, "B_FRAME_EVENTS") ? 1 : 0);
			chkFullUpdate->SetValue(strstr(m_Value, "B_FULL_UPDATE_ON_RESIZE") ? 1 : 0);
			chkNavigable->SetValue(strstr(m_Value, "B_NAVIGABLE") ? 1 : 0);
			chkNavJump->SetValue(strstr(m_Value, "B_NAVIGABLE_JUMP") ? 1 : 0);
			chkSubPixelPrecise->SetValue(strstr(m_Value, "B_SUBPIXEL_PRECISE") ? 1 : 0);
			
			UnlockLooper();
		}
	}
	
	virtual void OnApply(void)
	{
		// Apply/OK/Save button pressed. Convert view flags into 
		// string representation, and copy to m_Value.
		string s = "";
		
		if(chkWillDraw->Value()) 					s += "|B_WILL_DRAW";
		if(chkPulseNeeded->Value()) 			s += "|B_PULSE_NEEDED";
		if(chkFrameEvents->Value()) 			s += "|B_FRAME_EVENTS";
		if(chkFullUpdate->Value()) 				s += "|B_FULL_UPDATE_ON_RESIZE";
		if(chkNavigable->Value()) 				s += "|B_NAVIGABLE";
		if(chkNavJump->Value()) 					s += "|B_NAVIGABLE_JUMP";
		if(chkSubPixelPrecise->Value()) 	s += "|B_SUBPIXEL_PRECISE";

		memset(m_Value, 0, sizeof(m_Value));
		strncpy(m_Value, s.c_str(), s.length());
		
		if(m_Value[0] == '|')
		{
			int32 ilen = strlen(m_Value);
			char *tmp = new char[ilen + 2];
			strcpy(tmp, m_Value);
			strncpy(m_Value, ++tmp, ilen-1);
			m_Value[ilen-1] = 0;
			--tmp;
			delete [] tmp;
		}
		printf("%s\n", m_Value);
	}
	
	virtual bool OnMessage(BMessage *msg)
	{
		switch(msg->what)
		{
			default:
				return false;
		}
	}
	
	BCheckBox
		*chkWillDraw
	, *chkPulseNeeded
	, *chkFrameEvents
	, *chkFullUpdate
	,	*chkNavigable
	, *chkNavJump
	, *chkSubPixelPrecise
	;	
	
};

// ViewFlags property
bool TViewFlagsProperty::ModalEdit(TComponent **selected, uint32 count)
{
	bool result = true;
	char buf[T_PROP_VALUE_LENGTH] = "";
	strcpy(buf, Property()->Value());
	
	new ViewFlagsWin(this, buf, sizeof(buf), selected, count);
	
	return result;
}

// Font property
void TFontProperty::GetValuePickList(TStringList *list)
{
	list->MakeEmpty();
	list->Add("be_plain_font");
	list->Add("be_bold_font");
	list->Add("be_fixed_font");
}

// FontSize property
void TFontSizeProperty::GetValuePickList(TStringList *list)
{
	list->MakeEmpty();
	list->Add("6");
	list->Add("7");
	list->Add("8");
	list->Add("9");
	list->Add("10");
	list->Add("11");
	list->Add("12");
	list->Add("13");
	list->Add("14");
	list->Add("15");
	list->Add("16");
	list->Add("17");
	list->Add("18");
	list->Add("19");
	list->Add("20");
	list->Add("21");
	list->Add("22");
	list->Add("23");
	list->Add("24");
}

// TIntProperty
bool TIntProperty::Validate(const char *text)
{
	ClearValidateErrorMessage();
	if(!isnumeric(text))
	{
		SetValidateErrorMessage("The specified property value must be numeric.");
		return false;
	}
	return true;
}

bool TNameProperty::Validate(const char *text)
{
	return true;
}

// eof