//
// EventSource.h
//

#ifndef _EventSource_H
#define _EventSource_H

#include <Form.h>

#include <Rect.h>
#include <Control.h>

template <class T>
class TViewEventSource {

	// Event types
	typedef void (T::*TAllAttachedEvent)(void);
	typedef void (T::*TAttachedToWindowEvent)(void);
	typedef void (T::*TAllDetachedEvent)(void);
	typedef void (T::*TDetachedFromWindowEvent)(void);
	typedef void (T::*TDrawEvent)(BRect);
	typedef void (T::*TFrameMovedEvent)(BPoint);
	typedef void (T::*TFrameResizedEvent)(float, float);
	typedef void (T::*TGetPreferredSizeEvent)(float *, float *);
	typedef void (T::*TKeyDownEvent)(const char *, int32);
	typedef void (T::*TKeyUpEvent)(const char *, int32);
	typedef void (T::*TFocusEvent)(void);
	typedef void (T::*TMouseDownEvent)(BPoint);
	typedef void (T::*TMouseUpEvent)(BPoint);
	typedef void (T::*TMouseMovedEvent)(BPoint, uint32, const BMessage *);
	typedef void (T::*TPulseEvent)(void);
	typedef void (T::*TTargetedByScrollViewEvent)(BScrollView *);
	typedef void (T::*TActivatedEvent)(bool);
	typedef void (T::*TSetFontEvent)(const BFont *,uint32);

public:
	TViewEventSource<T>(T *parent)
	: OnAllAttached(0)
	, OnAttachedToWindow(0)
	, OnAllDetached(0)
	, OnDetachedFromWindow(0)
	, OnDraw(0)
	, OnFrameMoved(0)
	, OnFrameResized(0)
	, OnGetPreferredSize(0)
	, OnKeyDown(0)
	, OnKeyUp(0)
	, OnGotFocus(0)
	, OnLostFocus(0)
	, OnMouseDown(0)
	, OnMouseUp(0)
	, OnMouseMoved(0)
	, OnPulse(0)
	, OnTargetedByScrollView(0)
	, OnActivated(0)
	, OnSetFont(0)
	, m_Observer(parent)
	{}
	
	T *EventObserver(void) const { return m_Observer; }
	
	TAllAttachedEvent						OnAllAttached;
	TAttachedToWindowEvent			OnAttachedToWindow;
	TAllDetachedEvent						OnAllDetached;
	TDetachedFromWindowEvent		OnDetachedFromWindow;
	TDrawEvent									OnDraw;
	TFrameMovedEvent						OnFrameMoved;
	TFrameResizedEvent					OnFrameResized;
	TGetPreferredSizeEvent			OnGetPreferredSize;
	TKeyDownEvent								OnKeyDown;
	TKeyUpEvent									OnKeyUp;
	TFocusEvent								  OnGotFocus;
	TFocusEvent									OnLostFocus;
	TMouseDownEvent							OnMouseDown;
	TMouseUpEvent								OnMouseUp;
	TMouseMovedEvent						OnMouseMoved;
	TPulseEvent									OnPulse;
	TTargetedByScrollViewEvent	OnTargetedByScrollView;
	TActivatedEvent							OnActivated;
	TSetFontEvent								OnSetFont;
	
	// Event producer functions
	virtual	void	DoAllAttached() 
		{ if(OnAllAttached) (EventObserver()->*OnAllAttached)(); }
	virtual	void	DoAttachedToWindow() 
		{ if(OnAttachedToWindow) (EventObserver()->*OnAttachedToWindow)(); }
	virtual	void	DoAllDetached() 
		{ if(OnAllDetached) (EventObserver()->*OnAllDetached)(); }
	virtual	void	DoDetachedFromWindow() 
		{ if(OnDetachedFromWindow) (EventObserver()->*OnDetachedFromWindow)(); }
	virtual	void	DoDraw(BRect updateRect) 
	  { if(OnDraw) (EventObserver()->*OnDraw)(updateRect); }
	virtual	void	DoFrameMoved(BPoint new_position) 
		{ if(OnFrameMoved) (EventObserver()->*OnFrameMoved)(new_position); }
	virtual	void	DoFrameResized(float new_width, float new_height) 
		{ if(OnFrameResized) (EventObserver()->*OnFrameResized)(new_width, new_height); }
	virtual	void	DoKeyDown(const char *bytes, int32 numBytes)
		{ if(OnKeyDown) (EventObserver()->*OnKeyDown)(bytes, numBytes); }
	virtual	void	DoKeyUp(const char *bytes, int32 numBytes)
		{ if(OnKeyUp) (EventObserver()->*OnKeyUp)(bytes, numBytes); }
	virtual	void	DoMouseDown(BPoint where)
		{ if(OnMouseDown) (EventObserver()->*OnMouseDown)(where); }
	virtual	void	DoMouseUp(BPoint where)
		{ if(OnMouseUp) (EventObserver()->*OnMouseUp)(where); }
	virtual	void	DoMouseMoved(BPoint where,uint32 code,const BMessage *a_message)
		{ if(OnMouseMoved) (EventObserver()->*OnMouseMoved)(where, code, a_message); }
	virtual	void	DoPulse()
		{ if(OnPulse) (EventObserver()->*OnPulse)(); }
	virtual	void	DoTargetedByScrollView(BScrollView *scroll_view)
		{ if(OnTargetedByScrollView) (EventObserver()->*OnTargetedByScrollView)(scroll_view); }
	virtual	void	DoWindowActivated(bool state)
		{ if(OnActivated) (EventObserver()->*OnActivated)(state); }
	virtual void  DoSetFont(const BFont *font, uint32 mask = B_FONT_ALL)
		{ if(OnSetFont) (EventObserver()->*OnSetFont)(font,mask); }
private:
	T *m_Observer;	
};


#endif //_EventSource_H