//
// MenuEditor.cpp
//

#include <stdlib.h>

#include "MenuEditor.h"
#include "PostDispatchInvoker.h"
#include "Colors.h"

TMenuEditor::TMenuEditor(BOutlineListView *l, TItemsSavedEvent evt, void *receiver)
: BWindow(BRect(100,100,600,350), "Menu Items", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
, m_List(l)
, m_ItemsSavedEvent(evt)
, m_Receiver(receiver)
{
	SetFeel(B_MODAL_APP_WINDOW_FEEL);
	
	boxPanel = new BBox(Bounds());
	BRect r = Bounds();
	r.InsetBy(8,8);
	r.bottom -= 30;
	boxItems = new BBox(r, "boxItems");
	boxItems->SetLabel("Items:");
	
	AddChild(boxPanel);
	boxPanel->AddChild(boxItems);
	
	r.top = 15;
	r.bottom = r.top + 20;
	r.left = 20;
	r.right = 260;
	efText = new BTextControl(r,"efText","Item Text:","",NULL);
	efText->SetDivider(55);
	boxItems->AddChild(efText);
	
	r.top = 40;
	r.bottom = r.top + 20;
	r.left = 20;
	r.right = 260;
	efName = new BTextControl(r,"efName","Item Name:","",NULL);
	efName->SetDivider(efText->Divider());
	boxItems->AddChild(efName);

	// items outline
	r.left = boxItems->Bounds().Width()-200;
	r.top = 15;
	r.right = boxItems->Bounds().Width()-(25); // 10 + scrollbar width
	r.bottom = boxItems->Bounds().bottom-25;

	lvItems = new BOutlineListView(r,"lvItems",B_SINGLE_SELECTION_LIST);
	lvItems->SetSelectionMessage(new BMessage(LVITEMS_SELECTED));
	lvItems->SetInvocationMessage(new BMessage(LVITEMS_INVOKED));
	BScrollView *sv = new BScrollView("svItems", lvItems, B_FOLLOW_ALL, 0, true, true);
	rgb_color clr = {216,216,216,255};
	sv->SetViewColor(clr);
	boxItems->AddChild(sv);

	// shortcut properties
	r.top = efName->Frame().bottom + 10;
	r.left = efName->Frame().left;
	r.bottom = r.top + 20;
	r.right = r.left + 80;
	efShortcut = new BTextControl(r,"efShortcut","Shortcut:","",NULL);
	efShortcut->SetDivider(efText->Divider());
	efShortcut->TextView()->SetMaxBytes(1);
	boxItems->AddChild(efShortcut);
	
	r.top += 20;
	r.right = r.left + 120;
	BMenu *m = new BMenu("Select");
	m->AddItem(pmiShift = new BMenuItem("Shift", new BMessage(PMI_SHIFT)));
	m->AddItem(pmiControl = new BMenuItem("Control", new BMessage(PMI_CONTROL)));
	m->AddItem(pmiOption = new BMenuItem("Option", new BMessage(PMI_OPTION)));
	m->AddItem(pmiCommand = new BMenuItem("Command", new BMessage(PMI_COMMAND)));
	
	mfMods = new BMenuField(r,"mfMods","Modifiers:",m);
	mfMods->SetDivider(efText->Divider());
	mfMods->SetEnabled(false);
	boxItems->AddChild(mfMods);

	// Generate Event checkbox
	r.top += 30;
	r.right = r.left + 120;
	r.bottom = r.top + 20;
	chkEvent = new BCheckBox(r, "chkEvent", "Generate Event",0);
	chkEvent->SetValue(1);
	boxItems->AddChild(chkEvent);
	
	// Enabled checkbox
	r.top += 20;
	r.bottom = r.top + 20;
	chkEnabled = new BCheckBox(r, "chkEnabled", "Enabled",0);
	chkEnabled->SetValue(1);
	boxItems->AddChild(chkEnabled);

	// Marked checkbox
	r.top += 20;
	r.bottom = r.top + 20;
	chkMarked = new BCheckBox(r, "chkMarked", "Marked",0);
	chkMarked->SetValue(0);
	boxItems->AddChild(chkMarked);

	// item buttons
	r.right = efText->Frame().right;
	r.top = 80; r.left = (r.right - 80);
	r.bottom = r.top + 25;
	btnMenu = new BButton(r, "btnMenu", "Add Menu", new BMessage(BTNMENU_INVOKED));
	boxItems->AddChild(btnMenu);
	
	r.top += 30; r.bottom += 30;
	btnItem = new BButton(r, "btnItem", "Add Item", new BMessage(BTNITEM_INVOKED));
	boxItems->AddChild(btnItem);
		
	r.top += 30; r.bottom += 30;
	btnDelete = new BButton(r, "btnDelete", "Delete", new BMessage(BTNDELETE_INVOKED));
	boxItems->AddChild(btnDelete);

	r.top += 30; r.bottom += 30;
	btnUpdate = new BButton(r, "btnUpdate", "Update", new BMessage(BTNUPDATE_INVOKED));
	boxItems->AddChild(btnUpdate);
	
	btnMenu->SetEnabled(false);
	btnItem->SetEnabled(false);
	btnDelete->SetEnabled(false);
	btnUpdate->SetEnabled(false);
	
	// Save and Close buttons
	r.top = boxItems->Frame().bottom + 6;
	r.bottom = r.top + 25;
	r.left = (Bounds().Width()-140);
	r.right = Bounds().Width()-80;
	btnOK = new BButton(r, "btnOK", "Save", new BMessage(BTNOK_INVOKED)); 
	
	r.left += 70;
	r.right += 70;
	btnCancel = new BButton(r, "btnCancel", "Close", new BMessage(BTNCANCEL_INVOKED));

	boxPanel->AddChild(btnOK);
	boxPanel->AddChild(btnCancel);
	
	// set the edit controls up so that they tell when being edited.
	efText->TextView()->AddFilter(
		new PostDispatchInvoker(B_KEY_DOWN, new BMessage(EFTEXT_EDIT), this)
	);
	
	efShortcut->TextView()->AddFilter(
		new PostDispatchInvoker(B_KEY_DOWN, new BMessage(EFSHORTCUT_EDIT), this)
	);

	lvItems->SetTarget(this);
	btnMenu->SetTarget(this);
	btnItem->SetTarget(this);
	btnDelete->SetTarget(this);
	btnUpdate->SetTarget(this);
	btnOK->SetTarget(this);
	btnCancel->SetTarget(this);
	efText->SetTarget(this);
	efName->SetTarget(this);
	efShortcut->SetTarget(this);
	
	LoadMenu();
	Show();
}

void TMenuEditor::LoadMenu()
{
	lvItems->MakeEmpty();
	for(int32 i = 0; i < m_List->CountItems(); i++)
	{
		TDesignMenuItem *item = (TDesignMenuItem *)(m_List->ItemAt(i));
		if(item)
		{
			lvItems->AddItem(item);
		}
	}
}

void TMenuEditor::SaveMenu()
{
	m_List->MakeEmpty();
	for(int32 i = 0; i < lvItems->FullListCountItems(); i++)
	{
		TDesignMenuItem *item = dynamic_cast<TDesignMenuItem *>(lvItems->FullListItemAt(i));
		if(item) m_List->AddItem(item);
	}
	
	if(m_ItemsSavedEvent)
	{
		(*m_ItemsSavedEvent)(m_Receiver);
	}
}

BString TMenuEditor::MakeMenuItemName(TDesignMenuItem *item, const char *text)
{
	const char *abc = "_ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
	BString result = "";
	uint32 i = 0;
	uint32 itemCount = 0;
	
	// strip out all but valid characters
	if(text)
	{
		for(i = 0; i < strlen(text); i++)
		{
			if(strchr(abc, text[i]))
				result << text[i];
		}
	}
	
	// Get the parent item of this item, and prepend its ItemName
	// property onto the front of this name. This is so that, say, if 
	// there's an item captioned "Save As", under a "File" menu, it will be called
	// "FileSaveAs" by default, which is cool!
	if(item)
	{
		TDesignMenuItem *parent = dynamic_cast<TDesignMenuItem *>(lvItems->Superitem(item));
		if(parent)
		{
			BString s = parent->ItemName;
			s << result;
			result = s;
		}
	}
	
	// Make sure there are no duplicate item names
	for(i = 0; i < (uint32)lvItems->FullListCountItems(); i++)
	{
		TDesignMenuItem *item = dynamic_cast<TDesignMenuItem *>(lvItems->FullListItemAt(i));
		if(item)
		{
			BString tmp = result;
			tmp << itemCount;
			if(tmp == item->ItemName)
				++itemCount;
		}
	}
	
	if(itemCount) result << itemCount;
	return result;
}

void TMenuEditor::SetButtonState()
{
	// See if an item in the outline list is selected
	int32 index = lvItems->CurrentSelection();
	bool bIsSelected = (index >= 0);
	
	// If an item is selected, see if the selected
	// item is a separator
	bool bSeparatorSelected = false;
	bool bSubmenuSelected = false;
	bool bShortcutExists = false;
	if (bIsSelected) {
		TDesignMenuItem* pItem = dynamic_cast<TDesignMenuItem*>
			(lvItems->ItemAt(index));
		if (pItem) {
			bSeparatorSelected = (pItem->ItemType == dmiSeparator);
			bSubmenuSelected = (pItem->ItemType == dmiSubmenu);
			bShortcutExists = (pItem->Shortcut != "");
		}
	}
	
	if(bSeparatorSelected) printf("A separator is selected\n");

	// Delete: enable if anything is selected
	btnDelete->SetEnabled(bIsSelected);
	
	// Update: enable if anything is selected
	btnUpdate->SetEnabled(bIsSelected);
	
	// Add Item: enable if anything but a separator is selected	
	bool bEnableAddItem = bIsSelected && (! bSeparatorSelected);
	btnItem->SetEnabled(bEnableAddItem);

	// Add Menu: enable if there's any text in the label field
	const char* labelText = efText->Text();
	btnMenu->SetEnabled(labelText && (*labelText));
	
	// set the enabled state of the shortcut property controls.
	efShortcut->SetEnabled((!bSeparatorSelected)/* && (!bSubmenuSelected)*/);
	mfMods->SetEnabled((efShortcut->IsEnabled()) && (bShortcutExists));
	chkEvent->SetEnabled((!bSeparatorSelected));
	chkEnabled->SetEnabled(chkEvent->IsEnabled());
	chkMarked->SetEnabled(chkEvent->IsEnabled());
		
	// Add Item: text says Add Separator if button is enabled
	// and the label field contains separator text,
	// Add Item otherwise
	const char* itemText;
	if (bEnableAddItem && IsSeparator(labelText)) {
		itemText = STR_ADD_SEP;
	} else {
		itemText = STR_ADD_ITEM;
	}
	btnItem->SetLabel(itemText);

	// If add item button is enabled, it should be the default	
	if (bEnableAddItem) {
		btnItem->MakeDefault(true);
	} else {
		btnMenu->MakeDefault(true);
	}
}

void TMenuEditor::AddMenu(BMessage* message)
{
	const char* menuName = efText->Text();
	if ((! menuName) || (! *menuName)) {
		BAlert* pAlert = new BAlert("Add menu alert",
			"Please specify the menu name first.", "OK");
		pAlert->Go();
		return;
	}
	
	TDesignMenuItem *item = new TDesignMenuItem(menuName);
	
	item->ItemText = menuName;

	item->ItemName = efName->Text();
	if(item->ItemName == "")
		item->ItemName = MakeMenuItemName(0, menuName);

	item->ItemType = dmiSubmenu;
	
	lvItems->AddItem(item);	
	
	// Reset the label control and buttons
	efText->SetText("");
	efName->SetText("");
	efShortcut->SetText("");
	chkMarked->SetValue(false);
	
	SetButtonState();
}

void TMenuEditor::DeleteMenu(BMessage* message)
{
	int32 itemCount;	
	int32 selected = lvItems->CurrentSelection();
	if (selected < 0)
		return;
	
	TDesignMenuItem* pSelItem = dynamic_cast<TDesignMenuItem*>(lvItems->ItemAt(selected));
	if (! pSelItem)
		return;
	
	if (pSelItem->OutlineLevel() == 0) {
		// get index of item among items in 0 level
		itemCount = lvItems->CountItemsUnder(NULL, true);
		int32 i;
		for (i=0; i<itemCount; i++) {
			TDesignMenuItem *item = dynamic_cast<TDesignMenuItem *>(lvItems->ItemUnderAt(NULL, true, i));
			if (item == pSelItem) {
				break;
			}
		}
	}			
				
	// Find & record all subitems of selection, because we'll
	// have to delete them after they're removed from the list.
	BList subItems;
	int32 j;
	itemCount = lvItems->CountItemsUnder(pSelItem, false);
	for (j=0; j<itemCount; j++) {
		BListItem* pItem = lvItems->ItemUnderAt(pSelItem, false, j);
		subItems.AddItem(pItem);
	}
	
	// Note the superitem for reference just below
	TDesignMenuItem* pSuperitem = dynamic_cast<TDesignMenuItem*>
		(lvItems->Superitem(pSelItem));
		
	// Remove selected item and all subitems from
	// the outline list view.
	lvItems->RemoveItem(pSelItem); // removes super- and subitems
	
	// Delete the selected item and all subitems	
	for (j=0; j<itemCount; j++) {
		TDesignMenuItem *item = (TDesignMenuItem *)(subItems.ItemAt(j));
		delete item;
	}
	delete pSelItem; // deletes superitem
	
	if (pSuperitem) {
		// There's a bug in outline list view which does not
		// update the superitem correctly. The only way to do so
		// is to remove and add a brand-new superitem afresh.
		if (! lvItems->CountItemsUnder(pSuperitem, true))
		{
			int32 index = lvItems->FullListIndexOf(pSuperitem);
			lvItems->RemoveItem(pSuperitem);
			TDesignMenuItem* pCloneItem = new TDesignMenuItem(pSuperitem->Text(), pSuperitem->OutlineLevel());
			pCloneItem->CopyFrom(pSuperitem);
			lvItems->AddItem(pCloneItem, index);
			delete pSuperitem;
		}
	}
}

void TMenuEditor::AddMenuItem(BMessage* message)
{
	// Add item to outline list view but don't update the
	// window right away; the actual menu items will be
	// created dynamically later on.
	int32 selected = lvItems->CurrentSelection();
	if (selected >= 0) {
		TDesignMenuItem* item = dynamic_cast<TDesignMenuItem *>(lvItems->ItemAt(selected));
		if(item) {
			// This item is now a submenu if it wasn't before.
			item->ItemType = dmiSubmenu;
			
			int32 level = item->OutlineLevel() + 1;
			int32 index = lvItems->FullListIndexOf(item)+lvItems->CountItemsUnder(item, false) + 1;
			const char* itemName = efText->Text();
			bool bIsSeparator = IsSeparator(itemName);
			TDesignMenuItem *newItem = 0;
			
			if (bIsSeparator) {
				newItem = new TDesignMenuItem(STR_SEPARATOR, level);
				newItem->ItemType = dmiSeparator;
			} else {
				newItem = new TDesignMenuItem(itemName, level);
				newItem->ItemType = dmiItem;
			}
			
			newItem->ItemText = newItem->Text();
			
			newItem->Shortcut = efShortcut->Text();
			
			uint32 mods = 0;
			if(pmiShift->IsMarked()) mods |= B_SHIFT_KEY;
			if(pmiOption->IsMarked()) mods |= B_OPTION_KEY;
			if(pmiControl->IsMarked()) mods |= B_CONTROL_KEY;
			if(pmiCommand->IsMarked()) mods |= B_COMMAND_KEY;
			newItem->Modifiers = mods;
			
			newItem->MakeEvent = (chkEvent->Value() != 0);
			newItem->Enabled = chkEnabled->Value();
			newItem->Marked = chkMarked->Value();

			lvItems->AddItem(newItem, index);
			
			// Set the name of the item AFTER it's been added to the list view, so if it
			// needs to make its own name, it can use the name of it's parent menu item.
			newItem->ItemName = efName->Text();
			if(newItem->ItemName == "")
				newItem->ItemName = MakeMenuItemName(newItem, efText->Text());

			lvItems->Invalidate();	// outline view doesn't
															// invalidate correctly
			
			// Reset the label control and buttons
			efText->SetText("");
			efName->SetText("");
			efShortcut->SetText("");
			
			pmiShift->SetMarked(false);
			pmiOption->SetMarked(false);
			pmiControl->SetMarked(false);
			pmiCommand->SetMarked(false);
			
			SetButtonState();
		}
	}	
}

void TMenuEditor::UpdateItem(BMessage *message)
{
	int32 sel = lvItems->CurrentSelection();
	TDesignMenuItem *item = dynamic_cast<TDesignMenuItem *>(lvItems->ItemAt(sel));
	
	if(item)
	{
		item->ItemText = efText->Text();
		item->SetText(efText->Text());
		item->Shortcut = efShortcut->Text();
		
		item->ItemName = efName->Text();
		if(item->ItemName == "")
			item->ItemName = MakeMenuItemName(item, efText->Text());
			
		item->MakeEvent = chkEvent->Value();
		item->Enabled = chkEnabled->Value();
		item->Marked = chkMarked->Value();
		
		uint32 mods = 0;
		if(pmiShift->IsMarked()) mods |= B_SHIFT_KEY;
		if(pmiOption->IsMarked()) mods |= B_OPTION_KEY;
		if(pmiControl->IsMarked()) mods |= B_CONTROL_KEY;
		if(pmiCommand->IsMarked()) mods |= B_COMMAND_KEY;
		item->Modifiers = mods;
		
		lvItems->Invalidate();
	}
}

void TMenuEditor::MenuSelectionChanged(BMessage* message)
{
	int32 sel = lvItems->CurrentSelection();
	TDesignMenuItem *item = dynamic_cast<TDesignMenuItem *>(lvItems->ItemAt(sel));
	
	if(item)
	{
		efText->SetText((item->ItemType != dmiSeparator)? item->ItemText.String(): "");
		efName->SetText((item->ItemType != dmiSeparator)? item->ItemName.String(): "");
		efShortcut->SetText(item->ItemType == dmiItem? item->Shortcut.String(): "");
		
		uint32 mod = item->Modifiers;
		pmiControl->SetMarked(mod & B_CONTROL_KEY);
		pmiShift->SetMarked(mod & B_SHIFT_KEY);
		pmiCommand->SetMarked(mod & B_COMMAND_KEY);
		pmiOption->SetMarked(mod & B_OPTION_KEY);
		
		chkEvent->SetValue((item->ItemType != dmiSeparator)? item->MakeEvent: 0);
		chkEnabled->SetValue((item->ItemType != dmiSeparator)? item->Enabled:0);
		chkMarked->SetValue((item->ItemType != dmiSeparator)? item->Marked:0);
	}

	SetButtonState();
}

void TMenuEditor::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case EFTEXT_EDIT:
		{
			SetButtonState();
			break;
		}
		
		case EFSHORTCUT_EDIT:
		{
			mfMods->SetEnabled(strlen(efShortcut->Text()) > 0);
			break;
		}
		
	 	case BTNMENU_INVOKED:
		{
			AddMenu(message);
			break;
		}
		
		case BTNITEM_INVOKED:
		{
			AddMenuItem(message);
			break;
		}
		
		case BTNDELETE_INVOKED:
		{
			DeleteMenu(message);
			break;
		}
		
		case BTNUPDATE_INVOKED:
		{
			UpdateItem(message);
			break;
		}
		
	 	case BTNOK_INVOKED:
		{
			SaveMenu();
			break;
		}
		
	 	case BTNCANCEL_INVOKED:
		{
			PostMessage(B_QUIT_REQUESTED);
			break;
		}
		
		case LVITEMS_SELECTED:
		{
			MenuSelectionChanged(message);
			break;
		}
		
		case LVITEMS_INVOKED:
		{
			efText->MakeFocus();
			break;
		}
		
		case PMI_CONTROL:
		{
			pmiControl->SetMarked(!pmiControl->IsMarked());
			break;
		}
		
		case PMI_SHIFT:
		{
			pmiShift->SetMarked(!pmiShift->IsMarked());
			break;
		}

		case PMI_OPTION:
		{
			pmiOption->SetMarked(!pmiOption->IsMarked());
			break;
		}

		case PMI_COMMAND:
		{
			pmiCommand->SetMarked(!pmiCommand->IsMarked());
			break;
		}

		default:
		{
			BWindow::MessageReceived(message);
			break;
		}
	}
}

bool TMenuEditor::IsSeparator(const char* text) const
{
	if (! text) {
		return true;
	}
	
	if (! *text) {
		return true;
	}
	
	int32 len = strlen(text);
	for (int32 i = 0; i < len; i++) {
		char ch = text[i];
		if ((ch != ' ') && (ch != '-')) {
			return false;
		}
	}
	
	return true;
}

void TDesignMenuItem::DrawItem(BView *owner, BRect frame, bool complete)
{
//	BStringItem::DrawItem(owner, frame, complete);
//	return;

	const uint32 indent = 1;
	const rgb_color normalColor = {255,255,255,0};
	const rgb_color kEnabledFGColor = {0,0,0,255};
	const rgb_color kDisabledFGColor = {100,100,100,255};
	
	// If need to redraw the whole rect,
  if(IsSelected()) 
  { 
  	owner->SetHighColor(BeListSelectGrey);
  	owner->SetLowColor(BeListSelectGrey);
  } 
  else
  {
  	owner->SetHighColor(normalColor);
  	owner->SetLowColor(normalColor);
  }
  owner->FillRect(frame);
  
	// Draw the menu item text
	owner->MovePenTo(frame.left+(OutlineLevel()*indent)+4, frame.bottom-2); 
 	owner->SetHighColor((IsEnabled()) ? kEnabledFGColor: kDisabledFGColor); 
  owner->DrawString(Text());
  
	// Draw any shortcut keys that might exist
	if(Shortcut != "")
	{
		BString s = "Cmd+";
		
		if(Modifiers & B_CONTROL_KEY) s << "Alt+";
		if(Modifiers & B_SHIFT_KEY) s << "Shift+";
		if(Modifiers & B_OPTION_KEY) s << "Opt+";
		s << Shortcut;
		
		owner->MovePenTo(frame.right-60, frame.bottom-2);
		owner->DrawString(s.String());
	}
}

TDesignMenuItem *parentItem(BOutlineListView *list, TDesignMenuItem *item)
{
	TDesignMenuItem *result = 0;
	int32 index = list->IndexOf(item);
	
	if(index >= 0)
	{
		for(int32 i = index; i > 0; i--)
		{
			TDesignMenuItem *mi = (TDesignMenuItem *)list->ItemAt(i);
			if((mi) && (mi->Level < item->Level) && (mi->ItemType == dmiSubmenu))
			{
				result = mi;
				break;
			}
		}
	}
	return result;
}

BString ModifiersString(TDesignMenuItem *item)
{
	BString mods = "";
	if(item->Modifiers & B_SHIFT_KEY) mods << "|B_SHIFT_KEY";
	if(item->Modifiers & B_OPTION_KEY) mods << "|B_OPTION_KEY";
	if(item->Modifiers & B_CONTROL_KEY) mods << "|B_CONTROL_KEY";
	mods.RemoveFirst("|");
	return mods;
}

void EmitMenuItem(const char *parentName, TDesignMenuItem *mi, BString &s)
{
	s
	<< "  " << parentName << "->AddItem(" << mi->ItemName 
	<< " = new TMenuItem<T$FORMNAME>(this,\"" << mi->ItemText << "\""
	;
	
	if(mi->Shortcut != "")
	{
		s << ",'"<< mi->Shortcut << "'";
	}
	
	if(mi->Modifiers != 0)
	{
		BString mods = ModifiersString(mi);
		s << "," << mods;
	}
	
	s << "));\n";
	
	// Set the enabled/marked flags 
	s << "  " << mi->ItemName << "->SetEnabled(" << ((mi->Enabled)? "true":"false") << ");\n";
	s << "  " << mi->ItemName << "->SetMarked(" << ((mi->Marked)? "true":"false") << ");\n";
	
	// Assign the handler address if applicable
	if(mi->MakeEvent)
	{
		s << "  " << mi->ItemName << "->OnInvoked = &T$FORMNAME::" << mi->ItemName << "Invoked;\n";
	}
}

void EmitMenuItemsUnder(const char *parentName, BOutlineListView *list, TDesignMenuItem *item, BString &s)
{
	// initialize all of the items under this submenu
	for(int32 j = 0; j < list->CountItemsUnder(item, false); j++)
	{
		TDesignMenuItem *mi = dynamic_cast<TDesignMenuItem *>(list->ItemUnderAt(item,true,j));
		
		switch(mi->ItemType)
		{
			case dmiItem:
			{
				EmitMenuItem(item->ItemName.String(), mi, s);
				break;
			}
			
			case dmiSeparator:
			{
				s	<< "  " << item->ItemName << "->AddSeparatorItem();\n";
				break;
			}
			
			case dmiSubmenu:
			{
				s
				<< parentName << "->AddItem(" << mi->ItemName << " = new TMenu<T$FORMNAME>(this,\"" << mi->ItemText << "\"));\n"
				;
				EmitMenuItemsUnder(parentName,list,mi,s);
				break;
			}
		}
	}
}

void CreateMenuItem(TDesignMenuItem *mi, BMenu *menu)
{
	char shortcut = 0;
	uint32 mods = mi->Modifiers;
	
	if(mi->Shortcut != "")
	{
		shortcut = *(mi->Shortcut.String());
	}
	
	BMenuItem *mitem = new BMenuItem(mi->ItemText.String(), 0, shortcut, mods);
	menu->AddItem(mitem);
	mitem->SetEnabled(mi->Enabled);
	mitem->SetMarked(mi->Marked);
}

void CreateMenuItemsUnder(BOutlineListView *list, TDesignMenuItem *item, BMenu *menu)
{
	// initialize all of the items under this submenu
	for(int32 j = 0; j < list->CountItemsUnder(item, false); j++)
	{
		TDesignMenuItem *mi = dynamic_cast<TDesignMenuItem *>(list->ItemUnderAt(item,true,j));
		
		switch(mi->ItemType)
		{
			case dmiItem:
			{
				CreateMenuItem(mi, menu);
				break;
			}
			
			case dmiSeparator:
			{
				menu->AddSeparatorItem();
				break;
			}
			
			case dmiSubmenu:
			{
				BMenu *newMenu = new BMenu(mi->ItemText.String());
				menu->AddItem(newMenu);
				CreateMenuItemsUnder(list, mi, newMenu);
				break;
			}
		}
	}
}

void WriteMenuItemsTo(const char *keyname, BOutlineListView *list, BMessage *compInfo)
{
	BMessage msg('itmz');
	char buf[80] = "";
	BString 
  	  sModifiers
	, sType
	, sLevel
	;
	
	for(int32 i = 0; i < list->FullListCountItems(); i++)
	{
		sprintf(buf, "_item_%ld", i);
		
		TDesignMenuItem *item = (TDesignMenuItem *)(list->FullListItemAt(i));

		sModifiers = "";
		sType = "";
		sLevel = "";
		sModifiers << item->Modifiers;
		sLevel << item->OutlineLevel();
		
		switch(item->ItemType)
		{
			case dmiItem:				sType = "0";	break;
			case dmiSubmenu:		sType = "1";	break;
			case dmiSeparator:	sType = "2";	break;
		}
		
		msg.AddString(buf, item->ItemText.String());
		msg.AddString(buf, item->ItemName.String());
		msg.AddString(buf, item->Shortcut.String());
		msg.AddString(buf, sType.String());
		msg.AddString(buf, sModifiers.String());
		msg.AddString(buf, sLevel.String());
		msg.AddString(buf, item->MakeEvent? "true": "false");
		msg.AddString(buf, item->Enabled? "true": "false");
		msg.AddString(buf, item->Marked? "true": "false");
	}
	
	compInfo->AddMessage(keyname, &msg);
}

void ReadMenuItemsFrom(const char *keyname, BOutlineListView *list, BMessage *compInfo)
{
	BMessage msg('itmz');
	if(B_OK == compInfo->FindMessage(keyname, &msg))
	{
		list->MakeEmpty();
		
		char *name;
		uint32 type;
		int32 count;

		for(int32 i = 0; 
			(B_OK == msg.GetInfo(B_STRING_TYPE, i, &name, &type, &count)); 
			i++)
		{
			int32 x = 0;
			const char 
				*itemtext = 0
			, *itemname = 0
			, *shortcut = 0
			, *stype    = 0
			, *mods     = 0
			, *level    = 0
			, *makeEvent= 0
			, *enabled  = 0
			, *marked   = 0
			;
			
			msg.FindString(name, x++, &itemtext);
			msg.FindString(name, x++, &itemname);
			msg.FindString(name, x++, &shortcut);
			msg.FindString(name, x++, &stype);
			msg.FindString(name, x++, &mods);
			msg.FindString(name, x++, &level);
			msg.FindString(name, x++, &makeEvent);
			msg.FindString(name, x++, &enabled);
			msg.FindString(name, x++, &marked);
			
			TDesignMenuItem *item = new TDesignMenuItem(itemtext, atoi(level));
			item->ItemText = itemtext;
			item->ItemName = itemname;
			item->Shortcut = shortcut;
			
			switch(atoi(stype))
			{
				case 0: item->ItemType = dmiItem;			break;
				case 1: item->ItemType = dmiSubmenu;	break;
				case 2: item->ItemType = dmiSeparator;break;
				default:
					printf("Unknown value for Menu Item type!\n");
			}
			
			item->Modifiers = atoi(mods);
			item->Level = atoi(level);
			item->MakeEvent = (0 == strcmp(makeEvent, "true"));
			item->Enabled   = (0 == strcmp(enabled, "true"));
			item->Marked    = (0 == strcmp(marked, "true"));
			
			list->AddItem(item);
		}
	}
}

