//
// MenuEditor.h
//
#ifndef _MenuEditor_H
#define _MenuEditor_H

#include <InterfaceKit.h>
#include <String.h>
#include <stdio.h>

#define STR_SEPARATOR "------------------"
#define STR_ADD_SEP "Add Separator"
#define STR_ADD_ITEM "Add Item"

//
// Menu Item editor
//
enum TDesignMenuItemType {
	dmiItem, dmiSubmenu, dmiSeparator
};

class TDesignMenuItem: public BStringItem {
public:
  	TDesignMenuItem(const char *text, uint32 level = 0, bool expanded = true)
	: BStringItem(text, level, expanded)
	, ItemText(text)
	, ItemName("")
	, Shortcut("")
	, Trigger("")
	, ItemType(dmiItem)
	, Modifiers(0)
	, Level(level)
	, MakeEvent(false)
	, Enabled(true)
	, Marked(false)
	{}
	
	BString ItemText;
	BString ItemName;
	BString Shortcut;
	BString Trigger;
	TDesignMenuItemType ItemType;
	uint32 Modifiers;
	uint32 Level;
	bool MakeEvent;
	bool Enabled;
	bool Marked;
	
	TDesignMenuItem *CopyFrom(TDesignMenuItem *other)
	{
		if(other != this)
		{
			ItemText  = other->ItemText;
			ItemName  = other->ItemName;
			Shortcut  = other->Shortcut;
			Trigger   = other->Trigger;
			ItemType  = other->ItemType;
			Modifiers = other->Modifiers;
			Level 		= other->Level;
			MakeEvent	=	other->MakeEvent;
			Enabled   = other->Enabled;
			Marked    = other->Marked;
		}
		return this;
	}

	virtual void DrawItem(BView *owner, BRect frame, bool complete = false);
};

TDesignMenuItem *parentItem(BOutlineListView *list);
BString ModifiersString(TDesignMenuItem *item);
void EmitMenuItem(const char *, TDesignMenuItem *, BString &);
void EmitMenuItemsUnder(const char *, BOutlineListView *list, TDesignMenuItem *item, BString &s);
void CreateMenuItem(TDesignMenuItem *mi, BMenu *menu);
void CreateMenuItemsUnder(BOutlineListView *list, TDesignMenuItem *item, BMenu *menu);
void WriteMenuItemsTo(const char *keyname, BOutlineListView *list, BMessage *compInfo);
void ReadMenuItemsFrom(const char *keyname, BOutlineListView *list, BMessage *compInfo);

class TMenuEditor: public BWindow {
public:
  typedef void (*TItemsSavedEvent)(void *);

	enum {
	  	EFTEXT_EDIT = 0x1
		, EFSHORTCUT_EDIT
		, EFNAME_CHANGED
		, BTNMENU_INVOKED
		, BTNITEM_INVOKED
		, BTNDELETE_INVOKED
		, BTNUPDATE_INVOKED
		, BTNOK_INVOKED
		, BTNCANCEL_INVOKED
		, LVITEMS_SELECTED
		, LVITEMS_INVOKED
		, PMI_CONTROL
		, PMI_SHIFT
		, PMI_OPTION
		, PMI_COMMAND
	};
	
	BOutlineListView *m_List; // list of TDesignMenuItems
	TItemsSavedEvent m_ItemsSavedEvent;
	void *m_Receiver;
	
	TMenuEditor(BOutlineListView *, TItemsSavedEvent = 0, void * = 0);
	virtual void LoadMenu();
	virtual void SaveMenu();
	virtual BString MakeMenuItemName(TDesignMenuItem *, const char *text);
	virtual void MessageReceived(BMessage *);	
	
	void SetButtonState();
	void AddMenu(BMessage* message);
	void DeleteMenu(BMessage* message);
	void AddMenuItem(BMessage* message);
	void UpdateItem(BMessage *message);
	void MenuSelectionChanged(BMessage* message);
	bool IsSeparator(const char *text) const;
	
	BBox *boxPanel, *boxItems;
	BOutlineListView *lvItems;
	BButton 
		*btnMenu
	, *btnItem
	, *btnDelete
	, *btnUpdate
	,	*btnOK
	, *btnCancel
	;
	BTextControl 
		*efText
	, *efName
	, *efShortcut
	, *efTrigger
	;
	BMenuField *mfMods;
	BMenuItem
		*pmiShift
	, *pmiOption
	, *pmiControl
	, *pmiCommand
	;
	BCheckBox 
		*chkEvent
	, *chkEnabled
	, *chkMarked
	;
};

#endif // _MenuEditor_H
