//
// Menus.h
//

#ifndef _Menus_H
#define _Menus_H

#include <EventSource.h>

// Be includes
#include <Menu.h>
#include <MenuBar.h>
#include <MenuField.h>
#include <MenuItem.h>

//
// MenuItem
//
template <class T>
class TMenuItem: public BMenuItem, public TViewEventSource<T> {
	typedef BMenuItem inherited;
public:
	typedef void (T::*TInvokedEvent)(void);

	TMenuItem(
		T *owner
	, const char *label
	, char shortcut = 0
	, uint32 modifiers = 0
	)
	: BMenuItem(label, 0, shortcut, modifiers)
	, TViewEventSource<T>(owner)
	, OnInvoked(0)
	{}
	
	TInvokedEvent OnInvoked;
	
  virtual status_t Invoke(BMessage *msg = NULL)
  {
  	status_t result = inherited::Invoke(msg);
  	if(OnInvoked) {
  		(EventObserver()->*OnInvoked)();
  	}
  	return result;
  }
};


//
// Menu
//
template <class T>
class TMenu: public BMenu, public TViewEventSource<T> {
	typedef BMenu inherited;
public:
	typedef void (T::*TMenuPopupEvent)(void);
	
	TMenu(
		T *owner
	, const char *name
	, menu_layout layout = B_ITEMS_IN_COLUMN
	)
	: BMenu(name, layout)
	, TViewEventSource<T>(owner)
	{}
	
	TMenuPopupEvent OnPopup;
	
	virtual void Show(void)
	{
		inherited::Show();
		
		if(OnPopup) {
			(EventObserver()->*OnPopup)();
		}
	}
};

#endif // _Menus_H
